#######################################################################logger사용법#######################################################################
#main 모듈에서
#from NSX.log import CreateLogger
#def main():
#    logger.info('xxxx')
#
#if __name__ == '__main__':
#    logger = CreateLogger('NSX') #package이름으로 해야 로그파일이 남음.
#    main()
#
#연결된 모듈에서
#logger = logging.getLogger(__name__)
# def xxx
#     logger.info('xxxx')



import logging
from logging.handlers import RotatingFileHandler

def CreateLogger(logger_name):
    # Create Logger
    logger = logging.getLogger(logger_name)

    # Check handler exists
    if len(logger.handlers) > 0:
        return logger # Logger already exists

    logger.setLevel(logging.DEBUG)

    formatter = logging.Formatter('\n[%(levelname)s|%(name)s|%(filename)s:%(lineno)s] %(asctime)s > %(message)s')

    # Create Handlers
    #streamHandler = logging.StreamHandler()
    #streamHandler.setLevel(logging.DEBUG)

    fileMaxByte =  1024 * 1024 * 1
    fileHandler = RotatingFileHandler(f'{logger_name}.txt', mode='a', maxBytes = fileMaxByte,backupCount = 10)
    fileHandler.setLevel(logging.DEBUG)
    fileHandler.setFormatter(formatter)

    #logger.addHandler(streamHandler)
    logger.addHandler(fileHandler)

    return logger



from NSX import general_request
import time, sys, logging
logger = logging.getLogger(__name__)
def manager_state_check(Config_Data, Manager_Config):
    for Try_Count in range(40):
        API_result = manager_Service_check(Config_Data, Manager_Config["Address"], 'http')

        if API_result == 200:
            CTR_result = manager_Service_check(Config_Data, Manager_Config["Address"], 'controller')
        else:
            CTR_result = 400

        if CTR_result == 200 and Manager_Config['deployment'] == False:
            print('Wait 10 minutes for the system to stabilize.')
            logger.info('Wait 10 minutes for the system to stabilize.')
            time.sleep(600)
            return Manager_Config["Address"]
        elif CTR_result == 200 and Manager_Config['deployment'] == True:
            return Manager_Config["Address"]
        else:
            time.sleep(60)

        if Try_Count == 40:
            sys.exit('Manager deploy failed!!!')
        else:
            print(f'Try Count : {Try_Count}')

def manager_Service_check(Config_Data, mgr_address,Service):
    print('------------------------------------------------------------------------------------------------------')
    print(mgr_address, f'Manger {Service} Service ready check')
    print('------------------------------------------------------------------------------------------------------')
    for i in range(1,12):
        respond = general_request.request('GET',Config_Data,mgr_address,f'api/v1/node/services/{Service}/status')
        #respond = requests.get(f'https://{mgr_address}/api/v1/node/services/controller/status',headers=headers, auth=('admin', f'{Config_Data["NSX Manager 01 information"]["manager password"]}'), verify=False)

        if respond == 'respond_error' or respond == 'request_error':
            print(f'{Service} service is not ready. Retry after 60 seconds.')
            time.sleep(60)
        else:
            print(f'nsx-manager {Service} service respond : {respond["runtime_state"]}')

            #print(result)
            if respond['runtime_state'] == 'running':
                print('\033[32m' + f'{Service} service is ready.' + '\033[0m')
                logger.info(f'{mgr_address} Controller service is ready.')
                return 200
            else:
                logger.info(f'{Service} service service is not ready.')
                time.sleep(10)

    print('------------------------------------------------------------------------------------------------------')
    return 400

def manager_cluster(Config_Data, mgr_address):
    print('------------------------------------------------------------------------------------------------------')
    print('Manger Cluster ready check')
    print('------------------------------------------------------------------------------------------------------')

    for i in range(1,12):
        respond = general_request.request('GET',Config_Data,mgr_address,'api/v1/cluster/status')
        #respond = requests.get(f'https://{mgr_address}/api/v1/cluster/status',headers=headers, auth=('admin', f'{Config_Data["NSX Manager 01 information"]["manager password"]}'), verify=False,timeout=5)
        #print(respond)

        if respond == 'respond_error' or respond == 'request_error':
            print('System is not stable. Retry after 60 seconds.')
            time.sleep(60)
        else:
            cluster_status = respond["mgmt_cluster_status"]["status"]
            controller_Status = respond["control_cluster_status"]["status"]

            if cluster_status == 'STABLE':
                print('Manager Cluster Status : ','\033[32m' + cluster_status + '\033[0m')
            else:
                print('Manager Cluster Status : ',cluster_status)
            if controller_Status == 'STABLE':
                print('Controoler Cluster status : ','\033[32m' + controller_Status + '\033[0m')
            else:
                print('Controoler Cluster status : ',controller_Status)

            if cluster_status == 'STABLE' and controller_Status == 'STABLE':
                logger.info('cluster_status, controller_Status is stable.')
                return 200
            else:
                logger.info('cluster_status, controller_Status is not stable.')
                time.sleep(2)

    return 400

def BGP_status_check(Config_Data, T0_Router_Info, API_Manager_Address):
    for Try_Count in range(20):
        respond = general_request.request('GET',Config_Data,API_Manager_Address,f'policy/api/v1/infra/tier-0s/{T0_Router_Info["id"]}/locale-services/default/bgp/neighbors/status?enforcement_point_path=/infra/sites/default/enforcement-points/default')
        if respond == 'respond_error' or respond == 'request_error':
            print('Retry after 10 seconds.')
            time.sleep(10)
        else:
            print('------------------------------------------------------------------------------------------------------')
            print('T0 Router Name : ', T0_Router_Info['name'])
            return_T0_router_status = {'name':T0_Router_Info['name'],'status':[]}
            All_T0_Router_Status = 'success'
            if len(respond['results']) == 0:
                All_T0_Router_Status = 'fail'
            else:
                for T0_Rotuer_Status in respond['results']:
                    if 'type' in T0_Rotuer_Status:
                        print('------------------------------------------------------------------------------------------------------')
                        if T0_Rotuer_Status['connection_state'] != 'ESTABLISHED':
                            All_T0_Router_Status = 'fail'
                            print('T0 Router Neighbor Connection Status : ', '\033[31m' + T0_Rotuer_Status['connection_state'] + '\033[0m')
                        else:
                            print('T0 Router Neighbor Connection Status : ', '\033[32m' + T0_Rotuer_Status['connection_state'] + '\033[0m')

                        print('T0 Router Neighbor AS Number : ', T0_Rotuer_Status['remote_as_number'])
                        print('T0 Router Neighbor Address : ', T0_Rotuer_Status['neighbor_address'])
                        print('T0 Router Source Address : ', T0_Rotuer_Status['source_address'])

                        BGP_Neighbor_Status = {'connection_state': T0_Rotuer_Status['connection_state'], 'remote_as_number': T0_Rotuer_Status['remote_as_number'],'neighbor_address' : T0_Rotuer_Status['neighbor_address'], 'source_address': T0_Rotuer_Status['source_address']}
                        return_T0_router_status['status'].append(BGP_Neighbor_Status)
                    else:
                        All_T0_Router_Status = 'fail'

            if All_T0_Router_Status == 'success':
                print('\033[32m' + f'{T0_Router_Info["name"]} BGP Neighbors established.' + '\033[0m')
                logger.info(f'{T0_Router_Info["name"]} BGP Neighbor established. {return_T0_router_status}')
                #print(return_T0_router_status)
                return return_T0_router_status
            else:
                print('\033[31m' + f'Not all BGP Neighbor have established yet.' + '\033[0m')
                print('Retry after 60 seconds.')
                logger.warn(f'Not all BGP Neighbor have established yet.{return_T0_router_status}')
                time.sleep(60)
    return 'fail'

def compute_manager_check(Config_Data,API_Manager_Address,vCenter_Config):

    for Try_Count in range(12):
        VC_status = general_request.request('GET',Config_Data,API_Manager_Address,f'api/v1/fabric/compute-managers/{vCenter_Config["id"]}/status')

        if VC_status['connection_status'] == 'UP':
            print('\033[32m' + f'{vCenter_Config["ip_fqdn"]} vCenter Connection status UP.' + '\033[0m')
            logger.info(f'{vCenter_Config["ip_fqdn"]} vCenter Connection status UP.')
            return vCenter_Config['id']
        elif VC_status['connection_status'] == 'CONNECTING':
            print('\033[31m' + f'{vCenter_Config["ip_fqdn"]} vCenter Connection status Connecting. Wait 10 seconds.' + '\033[0m')
            logger.info(f'{vCenter_Config["ip_fqdn"]} vCenter Connection status Connecting. Wait 10 seconds.')
            time.sleep(10)
        elif VC_status['connection_status'] == 'DOWN':
            print('\033[31m' + f'{vCenter_Config["ip_fqdn"]} vCenter Connection status DOWN. Wait 10 seconds.' + '\033[0m')
            logger.info(f'{vCenter_Config["ip_fqdn"]} vCenter Connection status DOWN. Wait 10 seconds.')
            time.sleep(10)
        else:
            print(VC_status)

    sys.exit('vCenter Connection failed.')







import sys, time, logging
from NSX import general_request, object_search
logger = logging.getLogger(__name__)

def transportzone_create(Config_Data, TZ_info,API_Manager_Address):
    data = {
        "display_name":TZ_info["Name"],
        "transport_type":TZ_info["Traffic Type"].upper()
    }
    for Try_Count in range(10):
        respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/transport-zones', Request_Object_Type = 'Transport Zone', Request_Object_Name = TZ_info["Name"], Request_Data = data)
        #print(respond)
        if respond == 'respond_error' or respond == 'request_error':
            print('Retry after 30 seconds.')
            time.sleep(30)
        else:
            return respond
    sys.exit(f'{TZ_info["Name"]} Creation failed.')

def transportzone_add_team_policy(Config_Data, uplink_team, TZ_id,TZ_name,API_Manager_Address):
    respond = general_request.request('GET', Config_Data, API_Manager_Address,f'api/v1/transport-zones/{TZ_id}')

    if "uplink_teaming_policy_names" in respond:
        uplink_team_policy = respond['uplink_teaming_policy_names'].copy()

        if uplink_team['uplink_teaming_policy_name'] in uplink_team_policy:
            pass
        else:
            uplink_team_policy.append(uplink_team['uplink_teaming_policy_name'])

        if uplink_team_policy == respond['uplink_teaming_policy_names']:
            return 'not change'

    else:
        uplink_team_policy = [uplink_team['uplink_teaming_policy_name']]

    data = {
        "display_name": respond["display_name"],
        "transport_type":respond["transport_type"],
        "uplink_teaming_policy_names": uplink_team_policy,
        "_revision": respond["_revision"]
    }


    for Try_Count in range(10):
        respond = general_request.request('PUT', Config_Data, API_Manager_Address, f'api/v1/transport-zones/{TZ_id}', Request_Object_Type = 'Transport Zone Team Policy', Request_Object_Name = TZ_name, Request_Data = data)

        if respond == 'respond_error' or respond == 'request_error':
            print('Retry after 30 seconds.')
            time.sleep(30)
        else:
            #print(respond)
            return respond
    sys.exit(f'{TZ_name} updating fail.')

def make_uplink_list(uplink_list,port_type):

    test = uplink_list.split(",")
    return_uplink_list =[]
    for i in test:
        if i[0] == ' ':
            return_uplink_list.append({'uplink_name':i[1:],'uplink_type':port_type})
        elif i == 'n/a':
            pass
        else:
            return_uplink_list.append({'uplink_name':i,'uplink_type':port_type})
    return return_uplink_list

def uplink_profile_create(Config_Data,uplink_profile,API_Manager_Address):
    uplink_list ={}
    for i in uplink_profile:
        value = i.split(" ")
        if len(value) == 1:pass
        elif 'Teaming' == value[1]:
            uplink_list[i]={}
            uplink_list[i]['Active Uplinks'] = make_uplink_list(uplink_profile[i]['Active Uplinks'],'PNIC')
            uplink_list[i]['Standby Uplinks'] = make_uplink_list(uplink_profile[i]['Standby Uplinks'],'PNIC')
            if uplink_profile[i]['Teaming Policy'] == 'Load Balance Source':
                uplink_list[i]['Teaming Policy'] = 'LOADBALANCE_SRCID'
            elif uplink_profile[i]['Teaming Policy'] == 'Load Balance Source MAC Address':
                uplink_list[i]['Teaming Policy'] = 'LOADBALANCE_SRC_MAC'
            else:
                uplink_list[i]['Teaming Policy'] ='FAILOVER_ORDER'

    named_teaming = []
    for i in uplink_list:
        if i == 'Default Teaming':pass
        else:
            named_teaming.append({'name':uplink_profile[i]['Name'],'policy':uplink_list[i]['Teaming Policy'],'active_list':uplink_list[i]['Active Uplinks'],'standby_list':uplink_list[i]['Standby Uplinks']})

    data = {
        "teaming": {
            "policy": uplink_list['Default Teaming']['Teaming Policy'],
            "active_list": uplink_list['Default Teaming']['Active Uplinks'],
            "standby_list": uplink_list['Default Teaming']['Standby Uplinks']
        },
        "named_teamings":named_teaming,
        "transport_vlan": uplink_profile['Transport VLAN'],
        "resource_type": "UplinkHostSwitchProfile",
        "display_name": uplink_profile['Name'],
        #"mtu": uplink_profile['MTU']
    }
    #print(data)
    respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/host-switch-profiles', Request_Object_Type = 'Uplink Profile', Request_Object_Name = uplink_profile['Name'], Request_Data = data)
    return respond

def profile_list(Config_Data,profile_type):
    profile_listup = []
    for config_index in Config_Data:
        name = config_index.split(" ")

        if 'Profile' in name:
            profile_name =''
            for j in range(name.index('Profile')+1):
                profile_name = profile_name+" " +name[j]
            profile_name = profile_name[1:]

            if profile_name == profile_type:
                profile_listup.append(Config_Data[config_index])

            else:pass

    return profile_listup


def transport_node_profile(Config_Data,TNP_info,API_Manager_Address):

    Vds_id = object_search.id_finder(Config_Data, API_Manager_Address,'api/v1/fabric/virtual-switches', TNP_info["VDS Name"])
    Pool_id = object_search.id_finder(Config_Data, API_Manager_Address, 'api/v1/pools/ip-pools', TNP_info["TEP Pool"])
    Uplinks =[]
    #print(Vds_id, Pool_id)
    for key in TNP_info:
        if key == 'Uplinks':
            for Uplink in TNP_info[key]:
                Uplinks.append({'vds_uplink_name':TNP_info[key][Uplink],'uplink_name':Uplink})
    #print(Uplinks)

    for Uplink_profile in Config_Data['UPlink_profile_ids']:
        if TNP_info['Uplink Profile'] ==Uplink_profile['name']:
            Uplink_Profile_Id = Uplink_profile['id']
            break

    for TZ in Config_Data['TZ_ids']:
        if TNP_info['Transport Zone'] ==TZ['name']:
            TZ_id = TZ['id']
            break

    data = {
        "host_switch_spec":{
            "host_switches":[
                {
                    "host_switch_name":TNP_info["VDS Name"],
                    "host_switch_id":Vds_id,
                    "host_switch_type":"VDS",
                    "host_switch_mode":"STANDARD",
                    "host_switch_profile_ids":[
                        {
                            "key":"UplinkHostSwitchProfile",
                            "value":Uplink_Profile_Id
                        }
                    ],
                    "uplinks":Uplinks,
                    "ip_assignment_spec":{
                        "ip_pool_id":Pool_id,
                        "resource_type":"StaticIpPoolSpec"
                    },
                    "transport_zone_endpoints":[
                        {
                            "transport_zone_id":TZ_id
                        }
                    ]
                }
            ],
            "resource_type":"StandardHostSwitchSpec"
        },
        "resource_type":"TransportNodeProfile",
        "display_name":TNP_info["Name"]
    }
    #print(data)
    respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/transport-node-profiles', Request_Object_Type = 'Transport Node Profile', Request_Object_Name = TNP_info["Name"], Request_Data = data)
    #print(respond)
    #return respond

    if respond == 'respond_error' or respond == 'request_error':
        print('Retry after 30 seconds.')
        time.sleep(30)
    else:
        #print(respond)
        return respond
    sys.exit(f'{TNP_info["Name"]} updating fail.')

def transport_node_apply(Config_Data, TN_info, API_Manager_Address):
    Cluster_id = object_search.id_finder(Config_Data,API_Manager_Address,'api/v1/fabric/compute-collections',TN_info['Name'])

    for TNP_id in Config_Data['TNP_ids']:
        if TNP_id['name'] == TN_info['Transport Node Profile']:
            Apply_TNP_id = TNP_id['id']

    data = {
        "compute_collection_id": Cluster_id,
        "transport_node_profile_id": Apply_TNP_id
    }
    respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/transport-node-collections', Request_Object_Type = 'Apply to Transport Node Profile', Request_Object_Name = TN_info['Name'], Request_Data = data)
    return respond

def transport_node_state_check(Config_Data,API_Manager_Address, Resource_Type):

    for Try_Count in range(40):
        respond_TN_Status = general_request.request('GET', Config_Data, API_Manager_Address, 'api/v1/transport-nodes/state')
        respond_TN_Info = general_request.request('GET', Config_Data, API_Manager_Address, 'api/v1/transport-nodes')
        Retrun_TN_Staus = []
        #print(respond_TN_Status)
        all_state = 'success'
        for TN_Status in respond_TN_Status['results']:
            #print(TN_Status)
            result_TN_Status ={}
            for TN_Info in respond_TN_Info['results']:
                if Resource_Type == TN_Info['node_deployment_info']['resource_type']:
                    if TN_Status['transport_node_id'] == TN_Info['node_id'] :
                        print('------------------------------------------------------------------------------------------------------')
                        result_TN_Status = {'name':TN_Info['display_name'], 'node_id':TN_Info['node_id'],
                                            'node_deployment_state':TN_Status['node_deployment_state']['state'],
                                            'state':TN_Status['state']
                                            }
                        print(f'Node Name : {TN_Info["display_name"]}')
                        if TN_Status['state'] == 'success':
                            print('\033[32m' + f'Node State : {TN_Status["state"]}' + '\033[0m')
                        else:
                            print('\033[33m' + f'Node State : {TN_Status["state"]}' + '\033[0m')
                            all_state = 'fail'

                        if TN_Status['node_deployment_state']['state'] == 'success' or TN_Status['node_deployment_state']['state'] == 'NODE_READY':
                            print('\033[32m' + f'Node Deployment State : {TN_Status["node_deployment_state"]["state"]}' + '\033[0m')
                            Tep_ip = []
                            try:
                                for host_switch_states in TN_Status['host_switch_states']:
                                    if 'endpoints' in host_switch_states:
                                        for tep in host_switch_states['endpoints']:
                                            Tep_ip.append({'int_name':tep['device_name'], 'ip':tep['ip']})
                                            print(f'TEP IP : {tep["ip"]}')

                                    result_TN_Status['tep ip'] = Tep_ip
                            except:pass
                        else:
                            print('\033[33m' + f'Node Deployment State : {TN_Status["node_deployment_state"]["state"]}' + '\033[0m')
                            all_state='fail'
                        
            Retrun_TN_Staus.append(result_TN_Status)
        logger.info(f'Transport Node state : {Retrun_TN_Staus}')
        if all_state == 'success':
            return Retrun_TN_Staus
        else:
            print('Not all Transport nodes have succeeded yet.')
            print('Retry after 60 seconds.')
            time.sleep(60)
    return 'fail'

def global_config(Config_Data,global_config_info, API_Manager_Address):

    respond = general_request.request('GET', Config_Data, API_Manager_Address,f'api/v1/global-configs/SwitchingGlobalConfig')

    data = {
        "physical_uplink_mtu": Config_Data['Global Config']['MTU'],
        "resource_type": "SwitchingGlobalConfig",
        "_revision": respond['_revision']
    }


    respond = general_request.request('PUT', Config_Data, API_Manager_Address, 'api/v1/global-configs/SwitchingGlobalConfig', Request_Object_Type = 'Global Config', Request_Object_Name = 'Global Config', Request_Data = data)
    #print(respond)
    #return respond

    if respond == 'respond_error' or respond == 'request_error':
        print('Retry after 30 seconds.')
        time.sleep(30)
    else:
        #print(respond)
        return respond
    sys.exit(f'{TNP_info["Name"]} updating fail.')









import sys,time, argparse
from NSX import import_nsx_info, fw_multi_object
from NSX.log import CreateLogger

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-df','--distributed_firewall', action='store', dest='dfirewall', help='firewall config filename. csv file only. -df or -gf or -g or -s Required flag.')
    parser.add_argument('-gf','--gateway_firewall', action='store', dest='gfirewall', help='firewall config filename. csv file only. -df or -gf or -g or -s Required flag.')
    parser.add_argument('-g','--group', action='store', dest='group', help='inventory group config filename. csv file only. -f or -g or -s Required flag.')
    parser.add_argument('-s','--service', action='store', dest='service', help='inventory service config filename. csv file only. -f or -g or -s Required flag.')
    parser.add_argument('-c','--create', action='store_true', help='FW mode. must be select "create" or "update" or "delete"')
    parser.add_argument('-u','--update', action='store_true', help='FW update mode must be select "create" or "update" or "delete"')
    parser.add_argument('-d','--delete', action='store_true', help='FW update mode must be select "create" or "update" or "delete"')
    args = parser.parse_args()
    print(args)

    if args.dfirewall == None and args.gfirewall == None and args.service == None and args.group == None:
        sys.exit('Not enough flags')
    if args.create == True and args.update == True or args.create == True and args.delete == True or args.delete == True and args.update == True:
        sys.exit('Not Valid flags. Cannot be used at the same time.')
    elif args.create != True and args.update != True and args.delete != True:
        sys.exit('Not enough flags')
    elif args.create == True:
        API_MODE = 'create'
        print_text = 'Creating'
    elif args.update == True:
        API_MODE = 'update'
        print_text = 'Updating'
        print('\033[31m' + 'Changing the Firewall Configuration will disrupt the active traffic on the Firewall.' + '\033[0m')
        print('\033[31m' + 'Service Disruption is to be expected. ' + '\033[0m')
        contine_Value =  input('\033[31m' + 'Are you sure you want to continue(Y/N)? ' + '\033[0m')
        logger.warning(f'Changing the Load Balancer Configuration will disrupt the active traffic on the Firewall. Service Disruption is to be expected. Are you sure you want to continue(Y/N)? {contine_Value}')
        if contine_Value.upper() == 'Y':
            pass
        else:
            sys.exit('script finished.')
    elif args.delete == True:
        API_MODE = 'delete'
        print_text = 'Deleting'
        print('\033[31m' + 'Deleting the Firewall Configuration will STOP THE ACTIVE TRAFFIC on the Firewall.' + '\033[0m')
        contine_Value =  input('\033[31m' + 'Are you sure you want to continue(Y/N)? ' + '\033[0m')
        logger.warning(f'Changing the Firewall Configuration will  STOP THE ACTIVE TRAFFIC on the Firewall. Are you sure you want to continue(Y/N)? {contine_Value}')
        if contine_Value.upper() == 'Y':
            pass
        else:
            sys.exit('script finished.')
    else:
        pass

    for Try_Count in range(5): logger.info('NSX-T FW deploy START!!!!')

    Config_Data = {}
    print('******************************************************************************************************')
    print('NSX-T FW Config File Loading....')

    if args.group:
        Config_Data = import_nsx_info.import_csv2(args.group, 'Inventory_Groups')
        if 'NSX Manager General information' not in Config_Data:
            if 'NSX Manager General information' in Config_Data:
                Config_Data['NSX Manager General information'] = Config_Data['NSX Manager General information']
    if args.service:
        Config_Data = import_nsx_info.import_csv2(args.service, 'Inventory_Services')
        if 'NSX Manager General information' not in Config_Data:
            if 'NSX Manager General information' in Config_Data:
                Config_Data['NSX Manager General information'] = Config_Data['NSX Manager General information']

    if args.dfirewall:
        Config_Data = import_nsx_info.import_csv2(args.dfirewall, 'Distribute_Firewall')
        if 'NSX Manager General information' not in Config_Data:
            if 'NSX Manager General information' in Config_Data:
                Config_Data['NSX Manager General information'] = Config_Data['NSX Manager General information']

    if args.gfirewall:
        Config_Data = import_nsx_info.import_csv2(args.gfirewall, 'Gateway_Firewall')
        if 'NSX Manager General information' not in Config_Data:
            if 'NSX Manager General information' in Config_Data:
                Config_Data['NSX Manager General information'] = Config_Data['NSX Manager General information']

    print('File Loading is Complete.')
    logger.info(f'File Loading is Complete. {Config_Data}')
    print(Config_Data)

    print('******************************************************************************************************')
    print('NSX-T Firewall Object listup....')
    print(f'Firewall script mode : {API_MODE}')
    if 'Distribute_Firewall' in Config_Data:
        print(f'Distribute_Firewall_rules : {Config_Data["Distribute_Firewall"][0]}')
    if 'Gateway_Firewall' in Config_Data:
        print(f'Gateway_Firewall_rules : {Config_Data["Gateway_Firewall"][0]}')
    if 'Inventory_Groups' in Config_Data:
        print(f'Inventory_Groups : {Config_Data["Inventory_Groups"][0]}')
    if 'Inventory_Services' in Config_Data:
        print(f'Inventory_Services : {Config_Data["Inventory_Services"][0]}')

    sequence_list = []
    if API_MODE == 'create' or API_MODE == 'update':
        sequence_list.append('Inventory_Groups')
        sequence_list.append('Inventory_Services')
        sequence_list.append('Distribute_Firewall')
        sequence_list.append('Gateway_Firewall')
    elif API_MODE == 'delete':
        sequence_list.append('Gateway_Firewall')
        sequence_list.append('Distribute_Firewall')
        sequence_list.append('Inventory_Groups')
        sequence_list.append('Inventory_Services')

    for i in sequence_list:

        if i in Config_Data:
            print('******************************************************************************************************')
            print(f'NSX-T {i} {print_text}...')
            result = fw_multi_object.fw_multi_object(Config_Data,i,API_MODE)
            print(f'NSX-T {i} {print_text} Complete.')
            Config_Data[f'{i}_ids'] = result
            logger.info(f'NSX-T {i} {print_text} Complete. {result}')

if __name__ == '__main__':
    logger = CreateLogger('NSX')
    main()


import requests,json, sys, time
from NSX import object_search
from requests.packages.urllib3.exceptions import InsecureRequestWarning
import logging
logger = logging.getLogger(__name__)
#####################################################default value######################################################

headers = {'Content-Type': 'application/json'}
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

########################################################################################################################

def request_format(Request_Object_Type, API_MODE, Request_Method, Config_Data, object_Info, API_Manager_Address, url, data):
    for Try_Count in range(5):
        if API_MODE == 'create' or API_MODE == 'update':
            if 'id' in object_Info:
                respond = request(Request_Method, Config_Data, API_Manager_Address, url, Request_Object_Type = Request_Object_Type, Request_Object_Name = object_Info["Name"], Request_Data = data, API_MODE = 'update')
            else:
                respond = request(Request_Method, Config_Data, API_Manager_Address, url, Request_Object_Type = Request_Object_Type, Request_Object_Name = object_Info["Name"], Request_Data = data, API_MODE = 'create')
        elif API_MODE == 'delete':
            print(url+'/'+object_Info['id'])
            respond = request(Request_Method, Config_Data, API_Manager_Address, url+'/'+object_Info['id'], Request_Object_Type = Request_Object_Type, Request_Object_Name = object_Info["Name"], Request_Object_id = object_Info['id'], API_MODE = 'delete')

        if respond == 'respond_error' or respond == 'request_error':
            if API_MODE == 'delete':
                return  respond
            print('Retry after 10 seconds.')
            time.sleep(10)
        else:
            return respond
    sys.exit('LB Virtual service creating fail.')


def request(Request_Method, Config_Data, API_Manager_Address, Request_URL, **Request_Extension):
    try:
        session = requests.Session()
        #print(Request_Extension)
        if 'Add_header' in Request_Extension:
            headers.update(Request_Extension['Add_header'])

        session.headers = headers

        if 'auth' in Request_Extension:
            auth = Request_Extension['auth']

        else:
            auth = ('admin',Config_Data["NSX Manager General information"]["Password"])

        session.auth = auth
        if 'Request_Object_Type' in Request_Extension and Request_Extension['Request_Object_Type'] == 'T0-HA-VIP':
            Create_Request_URL = Request_URL
        elif 'Request_Object_Type' in Request_Extension and Request_Extension['Request_Object_Type'] == 'Static Route Bfd':
            Create_Request_URL = Request_URL
        elif Request_URL[:19] == 'policy/api/v1/infra':
            Create_Request_URL = 'policy/api/v1/infra'
        else:
            Create_Request_URL = Request_URL

        if 'API_MODE' in Request_Extension:
            if Request_Extension['API_MODE'] == 'delete':
                Create_Request_URL = Request_URL

        if 'Request_Data' in Request_Extension:
            logger.info(f'Request_Method - {Request_Method} | API Server - {API_Manager_Address} | Request_URL - {Request_URL} | Request_Data - {Request_Extension["Request_Data"]}')
            if Request_Method.upper()  == 'POST':
                respond = session.post(f'https://{API_Manager_Address}/{Create_Request_URL}',data=json.dumps(Request_Extension["Request_Data"]),verify=False)
            elif Request_Method.upper() == 'PATCH':
                respond = session.patch(f'https://{API_Manager_Address}/{Create_Request_URL}',data=json.dumps(Request_Extension["Request_Data"]),verify=False)
            elif Request_Method.upper() == 'PUT':
                respond = session.put(f'https://{API_Manager_Address}/{Create_Request_URL}',data=json.dumps(Request_Extension["Request_Data"]),verify=False)

            if respond.text:
                result = json.loads(respond.text)
                logger.info(f'Respond_Status_Code - {respond} | Respond_Data - {respond.text}' )
            else:
                logger.info(f'Respond_Status_Code - {respond} | NO_DATA' )
                result = respond

            if 'error_message' in result:
                print('\033[31m' + f'{Request_Extension["Request_Object_Name"]} {Request_Extension["Request_Object_Type"]} creation failed.' + '\033[0m')
                print('\033[31m' + 'ERROR MESSAGE : ', result['error_message'] + '\033[0m')
                logger.error(f'{Request_Extension["Request_Object_Name"]} {Request_Extension["Request_Object_Type"]} creation failed. ERROR MESSAGE : {result["error_message"]}')
                return 'respond_error'
            elif 200 <= respond.status_code < 300:
                if Request_Extension["Request_Object_Type"] == 'Distribute_Firewall' or Request_Extension["Request_Object_Type"] == 'Gateway_Firewall':
                    return 200
                if 'API_MODE' in Request_Extension:
                    respond = object_search.object_finder(Config_Data, [{'Name':Request_Extension["Request_Object_Name"], 'Object_type':Request_Extension["Request_Object_Type"]}], API_Manager_Address, Request_URL, True, API_MODE = Request_Extension['API_MODE'])
                else:
                    respond = object_search.object_finder(Config_Data, [{'Name':Request_Extension["Request_Object_Name"], 'Object_type':Request_Extension["Request_Object_Type"]}], API_Manager_Address, Request_URL, True)

                if 'id' in respond[0]:
                    return respond[0]['id']
                elif 'license_key' in respond[0]:
                    return respond[0]['registered']

        else:
            logger.debug(f'Request_Method - {Request_Method} | API Server - {API_Manager_Address} | Request_URL - {Request_URL}')
            if Request_Method.upper() == 'GET':
                respond = session.get(f'https://{API_Manager_Address}/{Request_URL}',auth=auth,headers=headers,verify=False)
                #print(respond.text)

            elif Request_Method.upper() == 'POST':
                respond = session.post(f'https://{API_Manager_Address}/{Create_Request_URL}',verify=False)

            elif Request_Method.upper() == 'DELETE':
                respond = session.delete(f'https://{API_Manager_Address}/{Create_Request_URL}',verify=False)


            if respond.text:
                try:
                    result = json.loads(respond.text)
                except:
                    result = respond
                logger.debug(f'Respond_Status_Code - {respond} | Respond_Data - {respond.text}')
            else:
                logger.info(f'Respond_Status_Code - {respond} | NO_DATA' )
                result = respond
            #print(result)
            if 'messages' in result:
                print('\033[31m' + 'ERROR MESSAGE : ', result['messages'][0]['default_message'] + '\033[0m')
                logger.error(f'ERROR MESSAGE : {result["messages"][0]["default_message"]}')
                return 'respond_error'
            if 'error_message' in result or respond.status_code >= 400:
                try:
                    print('\033[31m' + 'ERROR MESSAGE : ', result['error_message'] + '\033[0m')
                    logger.error(f'ERROR MESSAGE : {result["error_message"]}')
                except:
                    print('\033[31m' + 'ERROR MESSAGE : ' + '\033[0m', result)
                    logger.error(f'ERROR MESSAGE : {result}')

                return 'respond_error'
            elif 200 <= respond.status_code < 300:
                if 'API_MODE' in Request_Extension:
                    if Request_Extension['API_MODE'] == 'delete':
                        url =''
                        for object in Request_URL.split('/'):
                            if Request_Extension["Request_Object_id"] != object:
                                url = url + '/' + object
                        print(url)
                        result = object_search.object_finder(Config_Data, [{'Name':Request_Extension["Request_Object_Name"], 'Object_type':Request_Extension["Request_Object_Type"]}], API_Manager_Address, url[1:], True, API_MODE = Request_Extension['API_MODE'])
                return result
            else:
                print(respond.status_code, result)
                sys.exit('error ???')

    except requests.exceptions.Timeout as errd:
        print('\033[31m' + 'Timeout Error : ' + '\033[0m', errd)
        logger.error(f'request error {errd}')
        return 'request_error'

    except requests.exceptions.ConnectionError as errc:
        print('\033[31m' + "Error Connecting : " + '\033[0m', errc)
        logger.error(f'request error {errc}')
        return 'request_error'

    except requests.exceptions.HTTPError as errb:
        logger.error(f'request error {errb}')
        print('\033[31m' + "Http Error : " + '\033[0m', errb)
        return 'request_error'

    except requests.exceptions.RequestException as erra:
        logger.error(f'request error {erra}')
        print('\033[31m' + "AnyException : " + '\033[0m', erra)
        return 'request_error'



import subprocess, time, sys, ipaddress, logging
from NSX import vcenter_con, check_manager, general_request
logger = logging.getLogger(__name__)

def manager_deploy_main(Config_Data, Manager_Config, API_Manager_Address=None):
    print('******************************************************************************************************')
    print(f'{Manager_Config["Host Name"]} Manager deploying...')
    if Manager_Config['Deploy Type'] == 'Skip':
        print('Skip deploying this Manager.')
        return 0

    if Manager_Config['deployment'] == True:
        print('\033[33m' + f'{Manager_Config["Host Name"]} Manager already existed.' + '\033[0m')
        return Manager_Config['Address']
    elif Manager_Config['deployment'] == False:
        print(f'{Manager_Config["Host Name"]} Manager is not deployed.')
        if API_Manager_Address != None:
            if Manager_Config['Deploy Type'] == 'Manual':
                print('\033[31m' + f'Another manager is deployed. {Manager_Config["Host Name"]} has been set to manual deployment.' + '\033[0m')
                continue_value = input('If you want to proceed, try with auto deploy(Y/N)?')
                if continue_value.upper() == 'Y':
                    Manager_Config['Deploy Type'] = 'Auto'
                    logger.warning(f'Another manager is deployed. {Manager_Config["Host Name"]} has been set to manual deployment. If you want to proceed, try with auto deploy(Y/N)?? {continue_value}')
                else:
                    logger.warning(f'Another manager is deployed. {Manager_Config["Host Name"]} has been set to manual deployment. If you want to proceed, try with auto deploy(Y/N)? {continue_value}')
                    sys.exit('CHECK Your config csv file.')
            else:
                pass
        else:
            pass

        for Try_Count in range(4):
            if Try_Count == 0:
                print('******************************************************************************************************')
                print('Gathering a Session ID from vCenter.')
                for Config_index in Config_Data:
                    if Config_index[:7] == 'vCenter':
                        if Manager_Config["vcenter name"] == Config_Data[Config_index]["Name"]:
                            vc_session_id = vcenter_con.connect_to_api(Config_Data[Config_index])
                            #print(vc_session_id)
                            vc_address = Config_Data[Config_index]['ip or FQDN']
                            vc_user = Config_Data[Config_index]['Username']
                            vc_passwork = Config_Data[Config_index]['Password']
                            vc_host = Config_Data[Config_index]['deployment host']
                            break

            if Manager_Config['Deploy Type'] == 'Manual':
                create_result = ovftool_exec(Config_Data, Manager_Config, vc_address, vc_user, vc_passwork, vc_host)
            elif Manager_Config['Deploy Type'] == 'Auto':
                create_result = manager_auto_deploy(Config_Data, Manager_Config, vc_session_id, vc_address, API_Manager_Address)
            else:
                sys.exit('check your script.')

            if create_result == 'create':
                time.sleep(10)
                print('\033[32m' + Manager_Config["Host Name"], ' successfully created.' + '\033[0m')
                logger.info(f'{Manager_Config["Host Name"]} successfully created.')
                return Manager_Config['Address']
            elif Try_Count == 3:
                sys.exit('NSX Manager deploy failed.')
            else:
                print('------------------------------------------------------------------------------------------------------')
                print('NSX Manger deploy failed.')
                print('retry to deploying NSX Manager.')
                logger.info('NSX Manger deploy failed. retry to deploying NSX Manager')
                print('------------------------------------------------------------------------------------------------------')
        #print(create_result)

    #result = check_manager.manager_state_check(Config_Data,Manager_Config)
    #return result

def ovftool_exec(Config_Data, Manager_Config, vc_address, vc_user, vc_passwork, vc_host):
    ovftool_exec = f'{Config_Data["Deployment tool information"]["ovftool path"]}'
    ovf_command = [ovftool_exec]

    ovf_base_options = ['--sourceType=OVA',
                        '--X:injectOvfEnv',
                        '--acceptAllEulas','--powerOn',
                        '--noSSLVerify', '--allowExtraConfig',
                        f'--name={Manager_Config["Host Name"]}',
                        f'--datastore={Manager_Config["manager datastore"]}',
                        f'--network={Manager_Config["Portgroup"]}',
                        f'--diskMode={Manager_Config["manager disk mode"]}']

    ovf_command.extend(ovf_base_options)

    ovf_deployment_size = [f'--deploymentOption={Config_Data["NSX Manager General information"]["Size"]}']
    ovf_command.extend(ovf_deployment_size)

    if "Secondary Server" in Config_Data["DNS Servers"]:
        dns = f'{Config_Data["DNS Servers"]["Primary Server"]},{Config_Data["DNS Servers"]["Secondary Server"]}'

    else:
        dns = Config_Data["DNS Servers"]["Primary Server"]

    if "Secondary Server" in Config_Data["NTP Servers"]:
        ntp = f'{Config_Data["NTP Servers"]["Primary Server"]},{Config_Data["NTP Servers"]["Secondary Server"]}'

    else:
        ntp = Config_Data["NTP Servers"]["Primary Server"]

    ovf_ext_prop = [f'--prop:nsx_ip_0={Manager_Config["Address"]}',
                    '--prop:nsx_isSSHEnabled=True',
                    '--prop:nsx_allowSSHRootLogin=True',
                    '--prop:nsx_role=NSX Manager',
                    f'--prop:nsx_netmask_0={Manager_Config["Netmask"]}',
                    f'--prop:nsx_gateway_0={Manager_Config["Default Gateway"]}',
                    f'--prop:nsx_dns1_0={dns}',
                    f'--prop:nsx_domain_0={Manager_Config["Domain Name"]}',
                    f'--prop:nsx_ntp_0={ntp}',
                    f'--prop:nsx_passwd_0={Config_Data["NSX Manager General information"]["Password"]}',
                    f'--prop:nsx_cli_passwd_0={Config_Data["NSX Manager General information"]["Password"]}',
                    f'--prop:nsx_cli_audit_passwd_0={Config_Data["NSX Manager General information"]["Password"]}',
                    f'--prop:nsx_hostname={Manager_Config["Host Name"]}',
                    f'{Config_Data["Deployment tool information"]["ovf file path"]}/{Config_Data["NSX Manager General information"]["NSX Version and Build"]}']

    ovf_command.extend(ovf_ext_prop)

    vi_string = f'vi://{vc_user}:{vc_passwork}@{vc_address}/?ip={vc_host}'
    ovf_command.append(vi_string)
    logger.info(f'NSX Manager deploying from ovftool... | {ovf_command}')
    input = subprocess.run(ovf_command)
    if input.returncode == 0:
        return 'create'
    else:
        return 'failed'

def manager_auto_deploy(Config_Data, Manager_Config, vc_session_id, vc_address, API_Manager_Address):

    cluster_list = general_request.request('GET',Config_Data,vc_address,'api/vcenter/cluster',Add_header={'vmware-api-session-id':vc_session_id})
    host_list = general_request.request('GET',Config_Data,vc_address,'api/vcenter/host',Add_header={'vmware-api-session-id':vc_session_id})
    network_list = general_request.request('GET',Config_Data,vc_address,'api/vcenter/network',Add_header={'vmware-api-session-id':vc_session_id})
    datastore_list = general_request.request('GET',Config_Data,vc_address,'api/vcenter/datastore',Add_header={'vmware-api-session-id':vc_session_id})

    cluster_id = vcenter_con.find_object(cluster_list,Manager_Config['deployed cluster'],'cluster')
    host_id = vcenter_con.find_object(host_list,Manager_Config['deployed host'].lower(),'host')
    network_id = vcenter_con.find_object(network_list,Manager_Config['Portgroup'],'network')
    datastore_id = vcenter_con.find_object(datastore_list,Manager_Config['manager datastore'],'datastore')

    for vcenter in Config_Data['vc_ids']:
        if vcenter['name'] == Manager_Config['vcenter name']:
            vcenter_id = vcenter['id']

    prifix = ipaddress.IPv4Network(f'0.0.0.0/{Manager_Config["Netmask"]}').prefixlen

    if "Secondary Server" in Config_Data["DNS Servers"]:
        dns = [{Config_Data["DNS Servers"]["Primary Server"]},{Config_Data["DNS Servers"]["Secondary Server"]}]

    else:
        dns = [Config_Data["DNS Servers"]["Primary Server"]]

    if "Secondary Server" in Config_Data["NTP Servers"]:
        ntp = [{Config_Data["NTP Servers"]["Primary Server"]},{Config_Data["NTP Servers"]["Secondary Server"]}]

    else:
        ntp = [Config_Data["NTP Servers"]["Primary Server"]]
    data = {
        "deployment_requests": [
            {
                "roles": ["CONTROLLER", "MANAGER"],
                "form_factor": Config_Data["NSX Manager General information"]["Size"].upper(),
                "user_settings": {
                    "cli_password": Config_Data["NSX Manager General information"]["Password"],
                    "root_password": Config_Data["NSX Manager General information"]["Password"]
                },
                "deployment_config": {
                    "placement_type": "VsphereClusterNodeVMDeploymentConfig",
                    "vc_id": vcenter_id,
                    "management_network_id": network_id,
                    "hostname": Manager_Config["Host Name"],
                    "compute_id": cluster_id,
                    "host_id":host_id,
                    "storage_id": datastore_id,
                    "default_gateway_addresses":[
                        Manager_Config["Default Gateway"]
                    ],
                    "management_port_subnets":[
                        {
                            "ip_addresses":[
                                Manager_Config["Address"]
                            ],
                            "prefix_length": prifix
                        }
                    ],
                    "dns_servers": dns,
                    "ntp_servers":ntp,
                    "allow_ssh_root_login":"True",
                    "enable_ssh":"True"

                }
            }

        ]
    }
    #print(data)
    respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/cluster/nodes/deployments', Request_Object_Type = 'NSX Manager', Request_Object_Name = Manager_Config["Host Name"], Request_Data = data)

    if respond == 'request_error' or respond == 'request_error':
        sys.exit('manager auto deploy failed.')
    else:
        time.sleep(10)
        return 'create'


def ManagerClusterIP(Config_Data, API_Manager_Address):
    vip_address=Config_Data["Manager Cluster IP"]["Cluster IP"]

    for Try_Count in range(5):
        respond = general_request.request('GET',Config_Data,API_Manager_Address,'api/v1/cluster/api-virtual-ip')
        #print(respond)
        if respond == 'respond_error' or respond == 'request_error':
            print('Retry after 60 seconds.')
            time.sleep(60)
        else:
            break

    if respond['ip_address'] == '0.0.0.0':
        for Try_Count in range(20):
            respond = general_request.request('POST', Config_Data, API_Manager_Address, f'api/v1/cluster/api-virtual-ip?action=set_virtual_ip&ip_address={vip_address}', Request_Object_Type = 'Manager Cluster IP', Request_Object_Name = 'ip_address')
            #print(respond)

            if respond == 'respond_error' or respond == 'request_error' or respond['ip_address'] == '0.0.0.0':
                print('\033[31m' + 'Manager Cluster IP Creation failed.' + '\033[0m')
                print('Retry after 30 seconds.')
                logger.warning(f'Manager Cluster IP Creation failed.')
                time.sleep(30)
            elif Try_Count == 20:
                sys.exit('NSX-T Cluster Virtual IP crate fail')
            else:
                print('\033[32m' + 'Manager Cluster IP Creation success.' + '\033[0m')
                print('Wait 10 minutes for the system to stabilize.')
                logger.info('Manager Cluster IP Creation success. Wait 5 minutes for the system to stabilize.')
                time.sleep(600)
                break

    elif respond['ip_address'] == vip_address:
        print('\033[33m' + 'Manager Cluster IP already existed.' + '\033[0m')
        logger.info(f'Manager Cluster IP already existed. {respond["ip_address"]}')
        return respond['ip_address']
    else:
        sys.exit('NSX-T Cluster Virtual IP crate fail')

    print('Manager Virtual IP : ',respond['ip_address'])
    return respond['ip_address']


from NSX import general_request, check_manager
import time, ssl, socket, hashlib, sys, logging
logger = logging.getLogger(__name__)

def get_thumbprint(vCenter_Config):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(10)
    wrappedSocket = ssl.wrap_socket(sock)
    try:
        wrappedSocket.connect((vCenter_Config["ip or FQDN"], 443))
    except:
        module.fail_json(msg='Connection error while fatching thumbprint for server [%s].' % module.params['server'])
    else:
        der_cert_bin = wrappedSocket.getpeercert(True)
        pem_cert = ssl.DER_cert_to_PEM_cert(wrappedSocket.getpeercert(True))
        #print(pem_cert)

    thumb_sha256 = hashlib.sha256(der_cert_bin).hexdigest()
    wrappedSocket.close()
    # The API call expects the Thumbprint in Uppercase. While the API call is fixed,
    # below is a quick fix
    thumbprint = ""
    thumbprint = ':'.join(a+b for a,b in zip(thumb_sha256[::2], thumb_sha256[1::2]))

    return thumbprint

def compute_manager_regist(Config_Data,API_Manager_Address,vCenter_Config):
    thumbprint = get_thumbprint(vCenter_Config)

    data ={
        "display_name": vCenter_Config["Name"],
        "server": vCenter_Config["ip or FQDN"],
        "origin_type": "vCenter",
        "credential" : {
            "credential_type" : "UsernamePasswordLoginCredential",
            "username": vCenter_Config["Username"],
            "password": vCenter_Config["Password"],
            "thumbprint": thumbprint
        }
    }

    for Try_Count in range(5):
        respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/fabric/compute-managers', Request_Object_Type = 'Compute Manager', Request_Object_Name = vCenter_Config["Name"], Request_Data = data)
        if respond == 'respond_error' or respond == 'request_error':
            logger.warning(f'{vCenter_Config["Name"]} vCenter registering failed')
            print('Retry after 10 seconds.')
            time.sleep(10)
        else:
            time.sleep(30)
            print(respond)
            return respond
    sys.exit('vCenter Connection failed.')











import sys, time, logging
from NSX import object_search, policy, import_nsx_info
logger = logging.getLogger(__name__)

def lb_object_listup(Config_Data):
    LB_Object = {'Load_Balancer':[], 'Application_profile':[], 'Persistence_profile':[], 'Active_Health_monitor':[], 'Server_pool':[], 'Virtual_service':[], 'Certificate':[]}

    for object in Config_Data.keys():
        object_name = object[:len(object)-3]
        if 'Load Balancer' == object_name:
            LB_Object['Load_Balancer'].append(Config_Data[object])
        elif 'Application Profile'  == object_name:
            LB_Object['Application_profile'].append(Config_Data[object])
        elif 'Persistence Profile'  == object_name:
            LB_Object['Persistence_profile'].append(Config_Data[object])
        elif 'Active Health Monitor'  == object_name:
            LB_Object['Active_Health_monitor'].append(Config_Data[object])
        elif 'Server Pool'  == object_name:
            LB_Object['Server_pool'].append(Config_Data[object])
        elif 'Vritual Service'  == object_name:
            LB_Object['Virtual_service'].append(Config_Data[object])
        elif 'Certificate' == object_name:
            LB_Object['Certificate'].append(Config_Data[object])

    LB_Object['Load_Balancer'].insert(0,len(LB_Object['Load_Balancer']))
    LB_Object['Application_profile'].insert(0,len(LB_Object['Application_profile']))
    LB_Object['Persistence_profile'].insert(0,len(LB_Object['Persistence_profile']))
    LB_Object['Active_Health_monitor'].insert(0,len(LB_Object['Active_Health_monitor']))
    LB_Object['Server_pool'].insert(0,len(LB_Object['Server_pool']))
    LB_Object['Virtual_service'].insert(0,len(LB_Object['Virtual_service']))
    LB_Object['Certificate'].insert(0,len(LB_Object['Certificate']))
    LB_Object['NSX Manager General information'] = Config_Data['NSX Manager General information']

    return LB_Object

def lb_multi_object(Config_Data, object_name,API_MODE):
    import site
    NSX_API = import_nsx_info.import_csv(f'{site.getsitepackages()[1]}\\NSX\\NSX_API.csv')
    object_retrun = []
    API_Manager_Address = Config_Data['NSX Manager General information']['URL or IP']
    logger.info(f'{object_name} precheck request')

    object_listup = Config_Data[object_name]
    object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, NSX_API['NSX_API'][object_name],Request_Object_Type = object_name,API_MODE=API_MODE)
    object_re_listup.insert(0,object_listup[0])
    #print(object_re_listup)
    for i in range(object_re_listup[0]):
        request_value = False
        if API_MODE == 'create':
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                request_value = True
        if API_MODE == 'update':
            if 'id' in object_re_listup[i+1]:
                if object_name == 'Certificate':
                    print('\033[31m' + 'Certificate object update not support.' + '\033[0m')
                    logger.info('Certificate object update not support.')
                else:
                    request_value = True
            else:
                request_value = True
        if API_MODE == 'delete':
            if 'id' in object_re_listup[i+1]:
                request_value = True
            else:
                print('\033[33m' + f'Object {object_re_listup[i + 1]["Name"]} not found.' + '\033[0m')
                logger.warning(f'Object {object_re_listup[i + 1]["Name"]} not found.')

        if request_value == True:
            if object_name == 'Load_Balancer':
                object_id = policy.lb_service(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API['NSX_API'][object_name])
            elif object_name == 'Certificate':
                object_id = policy.certificate(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API['NSX_API'][object_name])
            elif object_name == 'Application_profile':
                object_id = policy.lb_application_profile(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API['NSX_API'][object_name])
            elif object_name == 'Persistence_profile':
                object_id = policy.lb_persistence_profile(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API['NSX_API'][object_name])
            elif object_name == 'Active_Health_monitor':
                object_id = policy.lb_active_monitor(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API['NSX_API'][object_name])
            elif object_name == 'Server_pool':
                object_id = policy.lb_pool(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API['NSX_API'][object_name])
            elif object_name == 'Virtual_service':
                object_id = policy.lb_virtual_service(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API['NSX_API'][object_name])

            object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_id})
    return object_retrun


from NSX import edge_deploy, compute_manager, policy, profile, manager_deploy, object_search, general_request,check_manager, license
import sys, time, logging
logger = logging.getLogger(__name__)

def multi_object_listup(Config_Data,object_name):
    object_listup = []
    object_count = 0
    if 'Segment attach' == object_name or 'Transport Zone Team Policy' == object_name:
        object_name = 'Segment'
    for config_index in Config_Data:
        try:
            compare_object = config_index[:len(object_name)]

            if object_name == compare_object:
                if config_index == 'NSX Manager General information':
                    pass
                else:
                    object_listup.append(Config_Data[config_index])
                    object_count = object_count + 1
            else: pass
        except: pass
    object_listup.insert(0, object_count)
    return object_listup

def multi_object_create(Config_Data,object_name, API_Manager_Address = None):
    logger.info(f'{object_name} precheck request')
    object_listup = multi_object_listup(Config_Data,object_name)
    #print(object_listup)
    object_retrun = []

    if object_name == 'vCenter':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'api/v1/fabric/compute-managers')
        object_re_listup.insert(0,object_listup[0])
        #print(object_re_listup)
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id'],'ip_fqdn':object_re_listup[i+1]['ip or FQDN']})
            else:
                vc_id = compute_manager.compute_manager_regist(Config_Data,API_Manager_Address,object_listup[i+1])
                object_retrun.append({'name':object_listup[i+1]['Name'],'id':vc_id,'ip_fqdn':object_re_listup[i+1]['ip or FQDN']})
        for i in range(object_re_listup[0]):
            check_manager.compute_manager_check(Config_Data,API_Manager_Address,object_retrun[i])
        return object_retrun

    elif object_name == 'NSX Manager':
        object_re_listup = object_search.manager_finder(Config_Data,object_listup[1:])
        object_re_listup.insert(0,object_listup[0])
        manager_create = False
        license_vc_regist = False
        first_manager_deploy = False
        for i in range(object_re_listup[0]):
            if object_re_listup[i+1]['deployment'] == True:
                continue_value = input('\033[33m' + f'{object_re_listup[i+1]["Host Name"]} Manager already existed. Next Step Continue(Y/N)?' + '\033[0m')
                logger.info(f'{object_re_listup[i+1]["Host Name"]} Manager already existed. Next Step Continue(Y/N)? {continue_value}')
                if continue_value.upper() == 'Y':
                    result = check_manager.manager_state_check(Config_Data,object_re_listup[i+1])
                    object_retrun.append({'name':object_listup[i + 1]['Host Name'],'ip':result})
                    API_Manager_Address = result
                    license_vc_regist = True
                    first_manager_deploy = True
                    break
                else:
                    sys.exit('script finsh')

        for i in range(object_re_listup[0]):
            if object_re_listup[i+1]['Address'] != API_Manager_Address:
                respond = manager_deploy.manager_deploy_main(Config_Data, object_listup[i + 1], API_Manager_Address)
                #print(object_listup[i + 1])
                if object_listup[i + 1]['Deploy Type'] != 'Skip' and object_re_listup[i+1]['deployment'] == False:
                    manager_create = True
                    if first_manager_deploy == False:
                        API_Manager_Address = respond
                        first_manager_deploy = True
                        license_vc_regist = True
                        manager_create = False
                        print('NSX-T Manager frist booting...')
                        print('Wait for a NSX-T Manager to be ready. Wait 10m')
                        time.sleep(600)
                        result = check_manager.manager_state_check(Config_Data,object_re_listup[i+1])
                        object_re_listup[i+1]['deployment'] = True
                object_retrun.append({'name':object_listup[i + 1]['Host Name'],'ip':respond})

            if license_vc_regist == True:
                print('******************************************************************************************************')
                print('NSX-T License Registering...')
                multi_object_create(Config_Data,'License',API_Manager_Address)
                print('Licenses registering is Complete.')
                logger.info('Licenses registering is Complete.')

                print('******************************************************************************************************')
                print('vCenter on NSX-T Registering...')
                Config_Data['vc_ids'] = multi_object_create(Config_Data,'vCenter',API_Manager_Address)
                print('vCenter registering is Complete.')
                logger.info(f'vCenter registering is Complete. {Config_Data["vc_ids"]}')
                license_vc_regist = False

            if i == 2 and manager_create == True:
                print('******************************************************************************************************')
                print('NSX-T Manager deploying and frist booting...')
                print('Wait for a NSX-T Manager to be ready. Wait 30m')
                time.sleep(1800)

        for i in range(object_re_listup[0]):
            if object_re_listup[i+1]['Address'] != API_Manager_Address and object_listup[i + 1]['Deploy Type'] != 'Skip':
                result = check_manager.manager_state_check(Config_Data,object_re_listup[i+1])

        return object_retrun

    elif object_name == 'License':
        object_listup_relist = []
        for i in range(object_listup[0]):
            object_listup_relist.append({'Name':list(object_listup[i+1].keys())[0],'license_key':object_listup[i+1][list(object_listup[i+1].keys())[0]]})
        object_re_listup = object_search.object_finder(Config_Data, object_listup_relist, API_Manager_Address, 'api/v1/licenses')
        #print(object_re_listup)
        object_re_listup.insert(0,object_listup[0])
        for i in range(object_re_listup[0]):
            if 'registered' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'registered':object_re_listup[i+1]['registered'],'license_key':object_re_listup[i+1]['license_key']})
            else:
                license_register = license.register_license(Config_Data,API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'registered':license_register,'license_key':object_re_listup[i+1]['license_key']})
        return object_retrun

    elif object_name == 'Manager Cluster IP':
        for manager in Config_Data['manager_ip']:
            if manager['ip']:
                API_Manager_Address =  manager['ip']
                break
        Cluseter_ip = manager_deploy.ManagerClusterIP(Config_Data, API_Manager_Address)
        Try_count = 0
        while True:
            result = check_manager.manager_cluster(Config_Data,Config_Data["Manager Cluster IP"]["Cluster IP"])

            if result == 200:
                break
            elif result == 400:
                time.sleep(60)
                Try_count = Try_count + 1

            if Try_count == 40:
                sys.exit('NSX-T Cluster Virtual IP create fail')
        return Cluseter_ip

    elif object_name == 'Global Config':
        golbal_config = profile.global_config(Config_Data,Config_Data['Global Config'],API_Manager_Address)

    elif object_name == 'Transport Zone':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'api/v1/transport-zones')
        object_re_listup.insert(0,object_listup[0])
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id'],'Traffic Type':object_re_listup[i+1]['Traffic Type']})
            else:
                TZ_id = profile.transportzone_create(Config_Data,object_re_listup[i+1],API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':TZ_id,'Traffic Type':object_re_listup[i+1]['Traffic Type']})
        return object_retrun

    elif object_name == 'Uplink Profile':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'api/v1/host-switch-profiles')
        object_re_listup.insert(0,object_listup[0])
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                Uplink_result = profile.uplink_profile_create(Config_Data,object_re_listup[i+1],API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':Uplink_result})
        return object_retrun

    elif object_name == 'IP Pool':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'policy/api/v1/infra/ip-pools')
        object_re_listup.insert(0,object_listup[0])
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                Ip_pool_id = policy.create_pool(Config_Data,object_re_listup[i+1],API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':Ip_pool_id})
        return object_retrun

    elif object_name == 'Transport Node Profile':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'api/v1/transport-node-profiles')
        object_re_listup.insert(0,object_listup[0])
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                TNP_id = profile.transport_node_profile(Config_Data,object_re_listup[i+1],API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':TNP_id})
        return object_retrun

    elif object_name == 'Host Transport Node':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'api/v1/transport-node-collections')
        object_re_listup.insert(0,object_listup[0])
        result_apply = False
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                host_TN = profile.transport_node_apply(Config_Data,object_re_listup[i+1],API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':host_TN})
                result_apply = True
        if result_apply == True:
            print('Transport Node Profile applying Complete.')
            print('Wait 10 minutes for NSX install.')
            time.sleep(600)
        return object_retrun

    elif object_name == 'Edge Node':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'api/v1/transport-nodes')
        object_re_listup.insert(0,object_listup[0])
        edge_create = False
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                edge_TN = edge_deploy.edge_auto_deploy(Config_Data, object_re_listup[i + 1],API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':edge_TN})
                edge_create = True
            if i%2 and edge_create:
                print('Edge Node deploying and frist booting...')
                print('Wait for a edge node to be ready. Wait 10m')
                time.sleep(600)
                edge_create = False
        return object_retrun

    elif object_name == 'Edge Cluster':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'api/v1/edge-clusters')
        object_re_listup.insert(0,object_listup[0])
        edge_cluster_create = False
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                edge_Cluster_id = edge_deploy.edge_cluster_create(Config_Data, object_re_listup[i + 1],API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':edge_Cluster_id})
                edge_cluster_create =True
        if edge_cluster_create :
            print('Wait 5 minutes for the system to stabilize.')
            time.sleep(300)
        return object_retrun

    elif object_name == 'Transport Zone Team Policy':
        for i in range(object_listup[0]):
            if 'Uplink Teaming Policy' in object_listup[i+1]:
                uplink_team = {"uplink_teaming_policy_name": object_listup[i+1]['Uplink Teaming Policy']}
                TZ_name = object_listup[i+1]['Transport Zone']
                for TZ in Config_Data['TZ_ids']:
                    if TZ['name'] == TZ_name:
                        TZ_id = TZ['id']
                respond = profile.transportzone_add_team_policy(Config_Data, uplink_team, TZ_id,TZ_name,API_Manager_Address)
                object_retrun.append({'name' : TZ_name, 'id': respond})
        return object_retrun

    elif object_name == 'Segment':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'policy/api/v1/infra/segments/')
        object_re_listup.insert(0,object_listup[0])
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                segment_id = policy.segment_create(Config_Data,object_re_listup[i+1], API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':segment_id})
                if object_re_listup[0] == i:
                    time.sleep(60)
        return object_retrun

    elif object_name == 'T0 Router':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'policy/api/v1/infra/tier-0s')
        object_re_listup.insert(0,object_listup[0])
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                T0_router_id = policy.T0_router_create(Config_Data,object_re_listup[i+1], API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':T0_router_id})
                if object_re_listup[0] == i:
                    time.sleep(60)
        return object_retrun

    elif object_name == 'BGP Neighbor':
        while True:
            for T0_Router_Info in Config_Data['T0_ids']:
                T0_Rotuer_Status = check_manager.BGP_status_check(Config_Data,T0_Router_Info,API_Manager_Address)
                object_retrun.append(T0_Rotuer_Status)
                if T0_Rotuer_Status == 'fail':
                    contunue_value = input(f'Check {T0_Router_Info["name"]} BGP Neighbor Status. Retry check BGP Status(Y/N/P=test env)?')
                    BGP_Status = 'fail'
                    break
                else:pass
            if T0_Rotuer_Status == 'fail' and contunue_value.upper() == 'Y':
                pass
            elif T0_Rotuer_Status == 'fail' and contunue_value.upper() == 'P':#테스트 환경에서만 사용할것
                return object_retrun
            elif T0_Rotuer_Status == 'fail' and contunue_value.upper() == 'N':
                sys.exit('BGP Neighbor Status fail.')
            else:
                return object_retrun

    elif object_name == 'T1 Router':
        object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, 'policy/api/v1/infra/tier-1s')
        object_re_listup.insert(0,object_listup[0])
        for i in range(object_re_listup[0]):
            if 'id' in object_re_listup[i+1]:
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
            else:
                T1_router_id = policy.T1_router_create(Config_Data,object_re_listup[i+1], API_Manager_Address)
                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':T1_router_id})
        return object_retrun

    elif object_name == 'Segment attach':
        for Try_Count in range(5):
            object_info_listup = general_request.request('GET', Config_Data, API_Manager_Address, 'policy/api/v1/infra/segments')

            if object_info_listup == 'respond_error' or object_info_listup == 'request_error':
                print('Retry after 10 seconds.')
                time.sleep(10)
            else:
                for i in range(object_listup[0]):
                    if 'Connected Virtual Router' in object_listup[i+1]:
                        for object_info in object_info_listup['results']:
                            if 'connectivity_path' in object_info and object_listup[i+1]['Name'] == object_info['display_name']:
                                print('\033[33m' + f'{object_listup[i+1]["Name"]} Segment Attach already created.' + '\033[0m')
                                logger.info(f'{object_listup[i+1]["Name"]} Segment Attach already created.')
                                segment_id = object_info['id']
                                #print(segment_id)
                            elif object_listup[i+1]['Name'] == object_info['display_name']:
                                segment_id = policy.segment_attach_router(Config_Data,object_listup[i+1], API_Manager_Address)
                        object_retrun.append({'name':object_listup[i+1]['Name'],'id':segment_id})
                return object_retrun


#python 3.9.4 nsx-t 3.1.2.0.0.17883596
#manager, edge 배포부분 개선 - 동시 배포로 수정 필요
from NSX import import_nsx_info, multi_object_create, check_manager, profile, manager_deploy
import sys, argparse
from NSX.log import CreateLogger

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-f','--file', action='store', dest='file', help='config filename')
    args = parser.parse_args()
    if args.file == None:
        args.file = 'config.csv'

    for Try_Count in range(5): logger.info('NSX-T deploy START!!!!')
    print('******************************************************************************************************')
    print('NSX-T Config File Loading....')
    Config_Data = import_nsx_info.import_csv(args.file)
    Config_Data['API_Manager_Address'] = None
    logger.info(f'File Loading is Complete. {Config_Data}')
    print('File Loading is Complete.')
    #print(Config_Data)

    print('******************************************************************************************************')
    print('NSX-T Manager Deploying...')
    Config_Data['manager_ip'] = multi_object_create.multi_object_create(Config_Data,'NSX Manager', Config_Data['API_Manager_Address'])
    print('NSX-T Manager deploying is Complete.')
    logger.info('NSX-T Manager deploying is Complete.')

    print('******************************************************************************************************')
    print('Manager Cluster IP Creating...')
    Config_Data['API_Manager_Address'] = multi_object_create.multi_object_create(Config_Data,'Manager Cluster IP', Config_Data['API_Manager_Address'])
    print('NSX-T Manager creating Complete.')
    logger.info(f'NSX-T Manager cluster creating is Complete. : {Config_Data["API_Manager_Address"]}')

    print('******************************************************************************************************')
    print('Global Configration Updating...')
    Config_Data['Global_Config'] = multi_object_create.multi_object_create(Config_Data, 'Global Config', Config_Data["API_Manager_Address"])
    print('Global Configration Updating Complete.')
    logger.info(f'Global Configration Updating is Complete. {Config_Data["Global_Config"]}')
    #print(Config_Data['TZ_ids'])

    print('******************************************************************************************************')
    print('Transport Zone Creating...')
    Config_Data['TZ_ids'] = multi_object_create.multi_object_create(Config_Data, 'Transport Zone', Config_Data["API_Manager_Address"])
    print('Transport Zone creating Complete.')
    logger.info(f'Transport Zone creating is Complete. {Config_Data["TZ_ids"]}')
    #print(Config_Data['TZ_ids'])

    print('******************************************************************************************************')
    print('TEP Pool Creating...')
    Config_Data['IP_pool_ids'] = multi_object_create.multi_object_create(Config_Data,'IP Pool', Config_Data["API_Manager_Address"])
    print('TEP Pool creating Complete.')
    logger.info(f'TEP Pool creating is Complete. {Config_Data["IP_pool_ids"]}')
    #print(Config_Data['IP_pool_ids'])

    print('******************************************************************************************************')
    print('Uplink Profile Creating...')
    Config_Data['UPlink_profile_ids'] = multi_object_create.multi_object_create(Config_Data,'Uplink Profile', Config_Data["API_Manager_Address"])
    print('Uplink Profile creating Complete.')
    logger.info(f'Uplink Profile creating is Complete. Config_Data["UPlink_profile_ids"]')
    #print(Config_Data['UPlink_profile_ids'])

    print('******************************************************************************************************')
    print('Transport Zone Team Policy Creating...')
    Config_Data['TZ_Team_ids'] = multi_object_create.multi_object_create(Config_Data, 'Transport Zone Team Policy', Config_Data["API_Manager_Address"])
    print('Transport Zone Team Policy creating Complete.')
    logger.info(f'Transport Zone Team Policy creating is Complete.')

    print('******************************************************************************************************')
    print('Transport Node Profile Creating...')
    Config_Data['TNP_ids'] = multi_object_create.multi_object_create(Config_Data,'Transport Node Profile',Config_Data["API_Manager_Address"])
    print('Transport Node Profile creating Complete.')
    logger.info(f'Transport Node Profile creating is Complete. {Config_Data["TNP_ids"]}')

    print('******************************************************************************************************')
    print('Transport Node Profile Apply...')
    Config_Data['Apply_TNP'] = multi_object_create.multi_object_create(Config_Data,'Host Transport Node',Config_Data["API_Manager_Address"])
    print('Transport Node Profile applying Complete.')
    logger.info(f'Transport Node Profile applying is Complete. {Config_Data["Apply_TNP"] }')

    print('******************************************************************************************************')
    print('Transport Node State check...')
    while True:
        respond = profile.transport_node_state_check(Config_Data,Config_Data["API_Manager_Address"], 'HostNode')
        if respond != 'fail': break
        else:
            continue_value = input('Check your Transport Nodes Status. try checking again(Y/N)?')

        if continue_value.upper() == 'Y':
            pass
        else:
            logger.info('Transport Node State checking is Failed.')
            sys.exit('Transport Node State checking FAILED.')

        Config_Data['TN_state'] = respond

        print('Transport Node State checking Complete.')
        logger.info(f'Transport Node State checking is Complete. {Config_Data["TN_state"]}')

    print('******************************************************************************************************')
    print('Edge node creating...')
    Config_Data['edge_ids'] = multi_object_create.multi_object_create(Config_Data,'Edge Node',Config_Data["API_Manager_Address"])
    print('Edge Node creating Complete.')
    logger.info(f'Edge Node creating is Complete. {Config_Data["edge_ids"]}')

    print('******************************************************************************************************')
    print('Edge Node State check...')
    while True:
        respond = profile.transport_node_state_check(Config_Data,Config_Data["API_Manager_Address"], 'EdgeNode')
        if respond != 'fail': break
        else:
            continue_value = input('Check your Edge Nodes Status. try checking again(Y/N)?')

        if continue_value.upper() == 'Y':
            pass
        else:
            logger.info('Edge Node State checking is Failed.')
            sys.exit('Edge Node State checking FAILED.')

        Config_Data['edge_state'] = respond

        print('Edge Node State checking Complete.')
        logger.info(f'Edge Node State checking is Complete. {Config_Data["edge_state"]}')

    print('******************************************************************************************************')
    print('Edge Cluster creating...')
    Config_Data['edge_cluster_ids'] = multi_object_create.multi_object_create(Config_Data,'Edge Cluster', Config_Data["API_Manager_Address"])
    print('Edge Cluster creating Complete.')
    logger.info(f'Edge Cluster creating is Complete. {Config_Data["edge_cluster_ids"]}')

    print('******************************************************************************************************')
    print('Segments creating...')
    Config_Data['segment_ids'] = multi_object_create.multi_object_create(Config_Data,'Segment', Config_Data["API_Manager_Address"])
    print('Segments creating Complete.')
    logger.info(f'Segments creating is Complete. {Config_Data["segment_ids"]}')

    print('******************************************************************************************************')
    print('Tier 0 Router creating...')
    Config_Data['T0_ids'] = multi_object_create.multi_object_create(Config_Data,'T0 Router', Config_Data["API_Manager_Address"])
    print('Tier 0 Router creating Complete.')
    logger.info(f'Tier 0 Router creating is Complete. {Config_Data["T0_ids"]}')

    print('******************************************************************************************************')
    print('Tier 0 Router BGP Neighbor checking...')
    Config_Data['BGP Neighbor'] = multi_object_create.multi_object_create(Config_Data,'BGP Neighbor', Config_Data["API_Manager_Address"])
    print('Tier 0 Router BGP Neighbor check is Complete.')
    logger.info(f'Tier 0 Router BGP Neighbor check is Complete. {Config_Data["BGP Neighbor"]}')

    print('******************************************************************************************************')
    print('Tier 1 Router creating...')
    Config_Data['T1_ids'] = multi_object_create.multi_object_create(Config_Data,'T1 Router', Config_Data["API_Manager_Address"])
    print('Tier 1 Router creating Complete.')
    logger.info(f'Tier 1 Router creating is Complete. {Config_Data["T1_ids"]}')

    print('******************************************************************************************************')
    print('Segments attaching to the router....')
    multi_object_create.multi_object_create(Config_Data,'Segment attach', Config_Data["API_Manager_Address"])
    print('Connecting Segments to the router Complete.')
    logger.info(f'Segments attaching is complete. {Config_Data["T1_ids"]}')

    print('******************************************************************************************************')
    for Try_Count in range(5): logger.info('NSX-T deploy FINISH!!!!')

if __name__ == '__main__':
    logger = CreateLogger('NSX')
    main()


#python 3.9.4 nsx-t 3.1.2.0.0.17883596
#ssl 기능 추가 필요
import sys,time, argparse
from NSX import import_nsx_info, lb_multi_object
from NSX.log import CreateLogger

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-f','--file', action='store', dest='file', help='config filename. csv file only. Required flag.')
    parser.add_argument('-c','--create', action='store_true', help='LB create mode. must be select "create" or "update" or "delete"')
    parser.add_argument('-u','--update', action='store_true', help='LB update mode must be select "create" or "update" or "delete"')
    parser.add_argument('-d','--delete', action='store_true', help='LB update mode must be select "create" or "update" or "delete"')
    args = parser.parse_args()
    #print(args)

    if args.file == None:
        sys.exit('Not enough flags')
    if args.create == True and args.update == True or args.create == True and args.delete == True or args.delete == True and args.update == True:
        sys.exit('Not Valid flags. Cannot be used at the same time.')
    elif args.create != True and args.update != True and args.delete != True:
        sys.exit('Not enough flags')
    elif args.create == True:
        API_MODE = 'create'
        print_text = 'Creating'
    elif args.update == True:
        API_MODE = 'update'
        print_text = 'Updating'
        print('\033[31m' + 'Changing the Load Balancer Configuration will disrupt the active traffic on the Load Balancer.' + '\033[0m')
        print('\033[31m' + 'Service Disruption is to be expected. ' + '\033[0m')
        contine_Value =  input('\033[31m' + 'Are you sure you want to continue(Y/N)? ' + '\033[0m')
        logger.warning(f'Changing the Load Balancer Configuration will disrupt the active traffic on the Load Balancer. Service Disruption is to be expected. Are you sure you want to continue(Y/N)? {contine_Value}')
        if contine_Value.upper() == 'Y':
            pass
        else:
            sys.exit('script finished.')
    elif args.delete == True:
        API_MODE = 'delete'
        print_text = 'Deleting'
        print('\033[31m' + 'Deleting the Load Balancer Configuration will STOP THE ACTIVE TRAFFIC on the Load Balancer.' + '\033[0m')
        contine_Value =  input('\033[31m' + 'Are you sure you want to continue(Y/N)? ' + '\033[0m')
        logger.warning(f'Changing the Load Balancer Configuration will  STOP THE ACTIVE TRAFFIC on the Load Balancer. Are you sure you want to continue(Y/N)? {contine_Value}')
        if contine_Value.upper() == 'Y':
            pass
        else:
            sys.exit('script finished.')
    else:
        pass

    for Try_Count in range(5): logger.info('NSX-T LB deploy START!!!!')

    print('******************************************************************************************************')
    print('NSX-T LB Config File Loading....')
    Config_Data = import_nsx_info.import_csv(args.file)
    logger.info(f'File Loading is Complete. {Config_Data}')
    print('File Loading is Complete.')

    print('******************************************************************************************************')
    print('NSX-T LB Object listup....')
    Config_Data = lb_multi_object.lb_object_listup(Config_Data)
    logger.info(f'NSX-T LB Object listup | {Config_Data}')
    print(f'LB script mode : {API_MODE}')
    print(f'Load Balancer : {Config_Data["Load_Balancer"][0]}')
    print(f'Application Profile : {Config_Data["Application_profile"][0]}')
    print(f'Persistence Profile : {Config_Data["Persistence_profile"][0]}')
    print(f'Active Health Monitor : {Config_Data["Active_Health_monitor"][0]}')
    print(f'Server Pool : {Config_Data["Server_pool"][0]}')
    print(f'Vritual Service : {Config_Data["Virtual_service"][0]}')
    print(f'Certificate : {Config_Data["Certificate"][0]}')

    sequence_list = []
    if API_MODE == 'create' or API_MODE == 'update':
        sequence_list.append('Certificate')
        sequence_list.append('Load_Balancer')
        sequence_list.append('Application_profile')
        sequence_list.append('Persistence_profile')
        sequence_list.append('Active_Health_monitor')
        sequence_list.append('Server_pool')
        sequence_list.append('Virtual_service')
    elif API_MODE == 'delete':
        sequence_list.append('Virtual_service')
        sequence_list.append('Server_pool')
        sequence_list.append('Active_Health_monitor')
        sequence_list.append('Application_profile')
        sequence_list.append('Persistence_profile')
        sequence_list.append('Load_Balancer')
        sequence_list.append('Certificate')

    for i in sequence_list:

        if Config_Data[i][0] != 0 :
            print('******************************************************************************************************')
            print(f'NSX-T {i} {print_text}...')
            result = lb_multi_object.lb_multi_object(Config_Data,i,API_MODE)
            print(f'NSX-T {i} {print_text} Complete.')
            Config_Data[f'{i}_ids'] = result
            logger.info(f'NSX-T {i} {print_text} Complete. {result}')

if __name__ == '__main__':
    logger = CreateLogger('NSX')
    main()


import csv
from NSX import general_request
def import_csv(csv_file):
    CSV_Data = csv.reader(open(csv_file,'r'))

    Config_Data = {}
    Config_index = ''
    Config_index2 = ''
    for line in CSV_Data:
        #print(line)
        if line[0] == '' and line[1] == '' and line[2] == '' and line[3] == '':
            pass
        if line[0] != '':
            Config_index = line[0]
            Config_Data[Config_index] = {}
        elif line[0] =='' and line[1] !='' and line[2] !='':
            Config_Data[Config_index][line[1]] = line[2]
        elif line[0] =='' and line[1] !='' and line[2] =='':
            Config_index2 = line[1]
            Config_Data[Config_index][Config_index2] = {}
        elif line[0] =='' and line[1] =='' and line[2] != '':
            Config_Data[Config_index][Config_index2][line[2]] = line[3]

    return Config_Data

def import_csv2(csv_file, Config_Mode):
    CSV_Data = csv.reader(open(csv_file,'r'))
    Config_Data = {next(CSV_Data)[0]:{'URL or IP':next(CSV_Data)[2],'Username':next(CSV_Data)[2],'Password':next(CSV_Data)[2]}}
    if Config_Mode == 'Inventory_Groups':
        Config_Data['Inventory_Groups'] = []

        for line in CSV_Data:
            if Config_Mode == 'Inventory_Groups':
                if line[0] == '' and line[1] == '':
                    pass
                elif line[0] == '':
                    Config_index['children'].append({'Type':line[1],'Key':line[2],'Operator':line[3],'Member':line[4]})
                elif line[0] != '' :
                    if line[1].upper() == 'IPADDRESS' or line[1].upper()[:6] == 'MEMBER':
                        Config_index = {'Name':line[0],'children':[]}
                        Config_index['children'].append({'Type':line[1],'Key':line[2],'Operator':line[3],'Member':line[4]})
                        Config_Data['Inventory_Groups'].append(Config_index)

        Config_Data['Inventory_Groups'].insert(0,len(Config_Data['Inventory_Groups']))
    elif Config_Mode == 'Inventory_Services':
        Config_Data['Inventory_Services'] = []

        for line in CSV_Data:
            if Config_Mode == 'Inventory_Services':
                if line[0] == '' and line[1] == '' and line[6] == '' or line[0].upper() == 'SERVICE NAME' and line[1].upper() == 'SERVICE ENTRY NAME':
                    pass
                elif line[0] == '':
                    Config_index['children'].append({'Name':line[1],'Protocol':line[2],'Src_Port':line[3],'Dst_Port':line[4],'Icmp_Type':line[5],'Entry_service':line[6]})
                elif line[0] != '' :
                    Config_index = {'Name':line[0],'children':[]}
                    Config_index['children'].append({'Name':line[1],'Protocol':line[2],'Src_Port':line[3],'Dst_Port':line[4],'Icmp_Type':line[5],'Entry_service':line[6]})
                    Config_Data['Inventory_Services'].append(Config_index)

        Config_Data['Inventory_Services'].insert(0,len(Config_Data['Inventory_Services']))

    elif Config_Mode == 'Distribute_Firewall' or Config_Mode == 'Gateway_Firewall':
        Config_Data[Config_Mode] = []
        for line in CSV_Data:
            #print(line)
            if Config_Mode == 'Distribute_Firewall' or Config_Mode == 'Gateway_Firewall':
                if line[0] == '' and line[2] == '':
                    pass
                elif line[0] == '' and line[2] != '':
                    Config_index['children'].append({'Name':line[2],'Source':line[3],'Destination':line[4],'Services':line[5],'Context_Profiles':line[6],'Applied_To':line[7], 'Action':line[8], 'Direction':line[9],'Logging':line[10]})
                elif line[0] != '' and line[2] == '':

                    Config_index = {'Name':line[0],'Applied_to':line[7],'Policy_category':line[1],'children':[]}
                    Config_Data[Config_Mode].append(Config_index)

        Config_Data[Config_Mode].insert(0,len(Config_Data[Config_Mode]))


    #print(Config_Data)
    return Config_Data


from setuptools import setup, find_packages

setup(name='seung-test',

      version='0.1.1',

      description='seung wang Test Package',

      author='Seung Wang Kim',

      author_email='seungwangkim@gmail.com',

      license='MIT',

      py_modules=['check_manager', 'compute_manager', 'edge_deploy', 'fw_multi_object', 'general_request', 'import_nsx_info', 'lb_multi_object', 'license', 'log', 'manager_deploy', 'multi_object_create', 'object_search', 'policy', 'profile', 'vcenter_con'],

      python_requires='>=3.9',

      #data_files=['NSX_API.csv', 'nsx_script_log.txt']

      #install_requires=[],

      #packages=['test']

      )


import sys, time, logging
from NSX import object_search, policy, import_nsx_info,general_request
logger = logging.getLogger(__name__)

def fw_multi_object(Config_Data, object_name,API_MODE):
    import site
    NSX_API = import_nsx_info.import_csv(f'{site.getsitepackages()[1]}\\NSX\\NSX_API.csv')
    object_retrun = []
    API_Manager_Address = Config_Data['NSX Manager General information']['URL or IP']
    logger.info(f'{object_name} precheck request')

    object_listup = Config_Data[object_name]
    object_re_listup = object_search.object_finder(Config_Data, object_listup[1:], API_Manager_Address, NSX_API['NSX_API'][object_name],Request_Object_Type = object_name,API_MODE=API_MODE)
    if object_name == 'Distribute_Firewall' or object_name == 'Gateway_Firewall':
        for object in object_re_listup:
            if 'children' in object and 'id' in object:
                object['children'] = object_search.object_finder(Config_Data, object['children'], API_Manager_Address, NSX_API['NSX_API'][object_name]+'/'+object['id'],Request_Object_Type = object_name+' rule',API_MODE=API_MODE)

    object_re_listup.insert(0,object_listup[0])

    print(object_re_listup)
    if object_name == 'Inventory_Groups' or object_name == 'Inventory_Services':
        for i in range(object_re_listup[0]):
            request_value = False
            if API_MODE == 'create':
                if 'id' in object_re_listup[i+1]:
                    object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_re_listup[i+1]['id']})
                else:
                    request_value = True
            if API_MODE == 'update':
                request_value = True
            if API_MODE == 'delete':
                if 'id' in object_re_listup[i+1]:
                    request_value = True
                else:
                    print('\033[33m' + f'Object {object_re_listup[i + 1]["Name"]} not found.' + '\033[0m')
                    logger.warning(f'Object {object_re_listup[i + 1]["Name"]} not found.')

            if request_value == True:
                if object_name == 'Inventory_Groups':
                    object_id = policy.inventory_group(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API)
                elif object_name == 'Inventory_Services':
                    object_id = policy.inventory_service(Config_Data,object_re_listup[i+1],API_Manager_Address,API_MODE,NSX_API)

                object_retrun.append({'name':object_re_listup[i+1]['Name'],'id':object_id})
    else:
        if object_name == 'Gateway_Firewall':
            object_ids = policy.gfw(Config_Data,object_re_listup,API_Manager_Address,API_MODE,NSX_API)
        elif object_name == 'Distribute_Firewall':
            object_ids = policy.dfw(Config_Data,object_re_listup,API_Manager_Address,API_MODE,NSX_API)
    return object_retrun



import requests,json,time, sys, logging
from NSX import general_request
logger = logging.getLogger(__name__)

def register_license(Config_Data,API_Manager_Address):

    for license in Config_Data['License']:
        data = {"license_key": Config_Data['License'][license]}

        for Try_Count in range(5):
            respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/licenses', Request_Object_Type = 'License', Request_Object_Name = 'License 01', Request_Data = data)
            if respond == 'respond_error' or respond == 'request_error':
                print('Retry after 10 seconds.')
                time.sleep(10)
            else:
                return respond
    sys.exit('Edge Node deploying fail.')

def get_license(Config_Data, API_Manager_Address):
    headers = {'Content-Type': 'application/json'}
    respond = requests.get(f'https://{API_Manager_Address}/api/v1/licenses',auth=('admin',f'{Config_Data["NSX Manager 01 information"]["manager password"]}'),headers=headers,verify=False)
    result = json.loads(respond.text)['results']
    retrun_license_list = []

    for license in result:
        retrun_license_list.append(license['license_key'])

    return retrun_license_list



from setuptools import setup, find_packages

setup(name='seung-test',

      version='0.1.1.8',

      description='This package is a Python script for deploying and configuring NSX-T TESTLAB. Made by seungwang.',

      author='Seung Wang Kim',

      author_email='seungwangkim@gmail.com',

      license='MIT',

      #py_modules=[],

      python_requires='>=3.9',

      package_data={'NSX':['*.*','data/*.*']},

      install_requires=['requests>=2.25.1'],

      packages=['NSX']

      )


from NSX import general_request
import logging
logger = logging.getLogger(__name__)

def connect_to_api(vCenter_config):
    auth = (vCenter_config['Username'],vCenter_config['Password'])
    respond = general_request.request('POST', None, vCenter_config["ip or FQDN"], 'api/session', Request_Object_Type = 'Request vCenter Session ID', Request_Object_Name = vCenter_config['Name'],auth=auth)
    #print ('vcenter-session-id : ',respond)
    return respond

def find_object(object_list, object_name,object_type):
    for object in object_list:
        if object['name'] == object_name:
            object_id = object[object_type]
            return object_id
        else:
            pass
    return 'object_non'





from NSX import vcenter_con, general_request, object_search, check_manager
import sys, time, re, ipaddress, logging
logger = logging.getLogger(__name__)
headers = {'Content-Type': 'application/json'}

def edge_auto_deploy(Config_Data, edge_info,API_Manager_Address):

    for Uplink_Profile in Config_Data['UPlink_profile_ids']:
        if Uplink_Profile['name'] == edge_info['Uplink Profile']:
            Uplink_Profile_id = Uplink_Profile['id']

    prefix = ipaddress.IPv4Network(f'0.0.0.0/{edge_info["Netmask"]}').prefixlen

    Physical_Nics = []
    i=0
    for Config_Index in Config_Data:
        Config_Index_split = Config_Index.split(' ')
        if Config_Index_split[0] == 'Uplink' :
            if Config_Data[Config_Index]['Name'] == edge_info['Uplink Profile']:
                Uplinks = re.split("[,\s]+",Config_Data[Config_Index]['Default Teaming']['Active Uplinks'])
                Uplinks.extend(re.split("[,\s]+",Config_Data[Config_Index]['Default Teaming']['Standby Uplinks']))

    for nic in Uplinks:
        if nic == 'n/a':
            pass
        else:
            Physical_Nics.append({"device_name":f'fp-eth{i}', "uplink_name":nic})
            i=i+1

    TZ_ids = []
    for TZ in Config_Data['TZ_ids']:
        edge_TZ_info_split = re.split("[,\n]",edge_info['Transport Zone'])
        for edge_TZ in edge_TZ_info_split:
            if TZ['name'] == edge_TZ:
                TZ_ids.append({'transport_zone_id':TZ['id']})


    for VC in Config_Data['vc_ids']:
        if VC['name'] == edge_info['Compute Manager']:
            vc_id = VC['id']

    for Config_index in Config_Data:
        if Config_index[:7] == 'vCenter':
            if edge_info["Compute Manager"] == Config_Data[Config_index]["Name"]:
                Vc_Session_id = vcenter_con.connect_to_api(Config_Data[Config_index])
                Vc_Address = Config_Data[Config_index]['ip or FQDN']
                break

    Pool_id = object_search.id_finder(Config_Data, API_Manager_Address, 'api/v1/pools/ip-pools', edge_info["TEP Pool"])

    Cluster_List = general_request.request('GET',Config_Data,Vc_Address,'api/vcenter/cluster',Add_header={'vmware-api-session-id':Vc_Session_id})
    Host_List = general_request.request('GET',Config_Data,Vc_Address,'api/vcenter/host',Add_header={'vmware-api-session-id':Vc_Session_id})
    Network_List = general_request.request('GET',Config_Data,Vc_Address,'api/vcenter/network',Add_header={'vmware-api-session-id':Vc_Session_id})
    Datastore_List = general_request.request('GET',Config_Data,Vc_Address,'api/vcenter/datastore',Add_header={'vmware-api-session-id':Vc_Session_id})

    #Cluster_List = vcenter_con.vcenter_search_info(Config_Data,Vc_Session_id,Vc_Address, 'cluster')
    Cluster_id = vcenter_con.find_object(Cluster_List,edge_info['Cluster Name'],'cluster')
    #Host_List = vcenter_con.vcenter_search_info(Config_Data,Vc_Session_id,Vc_Address, 'host')
    Host_id = vcenter_con.find_object(Host_List,edge_info['Deployed Host Name'].lower(),'host')
    #Datastore_List = vcenter_con.vcenter_search_info(Config_Data,Vc_Session_id,Vc_Address, 'datastore')
    Datastore_id = vcenter_con.find_object(Datastore_List,edge_info['Data Store'],'datastore')
    #Network_List = vcenter_con.vcenter_search_info(Config_Data,Vc_Session_id,Vc_Address, 'network')
    Mgmt_Net_id = vcenter_con.find_object(Network_List,edge_info['Management Portgroup'],'network')

    if "Secondary Server" in Config_Data["DNS Servers"]:
        DNS = [{Config_Data["DNS Servers"]["Primary Server"]},{Config_Data["DNS Servers"]["Secondary Server"]}]

    else:
        DNS = [Config_Data["DNS Servers"]["Primary Server"]]

    if "Secondary Server" in Config_Data["NTP Servers"]:
        NTP = [{Config_Data["NTP Servers"]["Primary Server"]},{Config_Data["NTP Servers"]["Secondary Server"]}]

    else:
        NTP = [Config_Data["NTP Servers"]["Primary Server"]]

    Network_ids = []
    for network in edge_info['Uplinks']:
        Network_ids.append(vcenter_con.find_object(Network_List,edge_info['Uplinks'][network],'network'))

    '''
    #for nsx-t 3.1.2
    data = {
        "host_switch_spec":{
            "host_switches":[
                {
                    "host_switch_name":"nsxvswitch",
                    "host_switch_mode":"STANDARD",
                    "host_switch_profile_ids":[
                        {
                            "key":"UplinkHostSwitchProfile",
                            "value":Uplink_Profile_id
                        }
                    ],
                    "pnics":Physical_Nics,
                    "ip_assignment_spec":{
                        "ip_pool_id":Pool_id,
                        "resource_type":"StaticIpPoolSpec"
                    },
                    "transport_zone_endpoints":TZ_ids
                }
            ],
            "resource_type":"StandardHostSwitchSpec"
        },
        "transport_zone_endpoints": [],
        "node_deployment_info":{
            "deployment_config":{
                "vm_deployment_config":{
                    "vc_id":vc_id,
                    "compute_id":Cluster_id,
                    "host_id":Host_id,
                    "storage_id":Datastore_id,
                    "management_network_id":Mgmt_Net_id,
                    "hostname":edge_info["Name"],
                    "management_port_subnets": [
                        {
                            "ip_addresses": [
                                edge_info['Management ip']
                            ],
                            "prefix_length": prefix
                        }
                    ],
                    "default_gateway_addresses": [
                        edge_info['Gateway']
                    ],
                    "data_network_ids":Network_ids,
                    "search_domains":[edge_info["Domain Name"]],
                    "dns_servers": DNS,
                    "ntp_servers": NTP,
                    "enable_ssh": "true",
                    "allow_ssh_root_login": "true",
                    "placement_type":"VsphereDeploymentConfig"
                },
                "form_factor":edge_info['Size'].upper(),
                "node_user_settings":{
                    "root_password":edge_info['Password'],
                    "cli_password":edge_info['Password']
                }
            },
            "resource_type":"EdgeNode"
        },
        "resource_type":"TransportNode",
        "display_name":edge_info["Name"]
    }
    '''
    data = {
        "host_switch_spec":{
            "host_switches":[
                {
                    "host_switch_name":"nsxvswitch",
                    "host_switch_mode":"STANDARD",
                    "host_switch_profile_ids":[
                        {
                            "key":"UplinkHostSwitchProfile",
                            "value":Uplink_Profile_id
                        }
                    ],
                    "pnics":Physical_Nics,
                    "ip_assignment_spec":{
                        "ip_pool_id":Pool_id,
                        "resource_type":"StaticIpPoolSpec"
                    },
                    "transport_zone_endpoints":TZ_ids
                }
            ],
            "resource_type":"StandardHostSwitchSpec"
        },
        "node_deployment_info":{
            "deployment_config":{
                "vm_deployment_config":{
                    "vc_id":vc_id,
                    "compute_id":Cluster_id,
                    "host_id":Host_id,
                    "storage_id":Datastore_id,
                    "management_network_id":Mgmt_Net_id,
                    "management_port_subnets": [
                        {
                            "ip_addresses": [
                                edge_info['Management ip']
                            ],
                            "prefix_length": prefix
                        }
                    ],
                    "default_gateway_addresses": [
                        edge_info['Gateway']
                    ],
                    "data_network_ids":Network_ids,
                    "placement_type":"VsphereDeploymentConfig"
                },
                "form_factor":edge_info['Size'].upper(),
                "node_user_settings":{
                    "root_password":edge_info['Password'],
                    "cli_password":edge_info['Password']
                }
            },
            "node_settings":{
                "hostname":edge_info["Name"],
                "search_domains":[edge_info["Domain Name"]],
                "dns_servers":DNS,
                "ntp_servers":NTP,
                "enable_ssh":"true",
                "allow_ssh_root_login":"true"
            },
            "resource_type":"EdgeNode"
        },
        "resource_type":"TransportNode",
        "display_name":edge_info["Name"]
    }
    for Try_Count in range(5):
        respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/transport-nodes/', Request_Object_Type = 'Edge Node', Request_Object_Name = edge_info["Name"], Request_Data = data)
        #print(respond)
        if respond == 'respond_error' or respond == 'request_error':
            print('Retry after 30 seconds.')
            time.sleep(30)
        else:
            time.sleep(10)
            return respond
    sys.exit('Edge Node deploying fail.')

def edge_cluster_create(Config_Data, edge_cluster_info, API_Manager_Address):
    if 'Edge Clusetr Profile' in edge_cluster_info:
        Edge_Cluster_Profile = edge_cluster_info['Edge Clusetr Profile']
    else:
        Edge_Cluster_Profile = 'nsx-default-edge-high-availability-profile'

    Edge_Cluster_Profile_id = object_search.id_finder(Config_Data,API_Manager_Address,'api/v1/cluster-profiles',Edge_Cluster_Profile)

    cluster_members=[]
    cluster_member_split = re.split("[,\s]+",edge_cluster_info['Members'])
    for edge_id in Config_Data['edge_ids']:
        for edge_name in cluster_member_split:
            if edge_name == edge_id['name']:
                cluster_members.append({'transport_node_id' : edge_id['id']})

    data = {
        "display_name" : edge_cluster_info['Name'],
        "cluster_profile_bindings": [
            {
                "profile_id":Edge_Cluster_Profile_id,
                "resource_type": "EdgeHighAvailabilityProfile"
            }
        ],
        "members":  cluster_members
    }
    #print(data)
    for Try_Count in range(5):
        respond = general_request.request('POST', Config_Data, API_Manager_Address, 'api/v1/edge-clusters', Request_Object_Type = 'Edge Cluster', Request_Object_Name = edge_cluster_info['Name'], Request_Data = data)
        if respond == 'respond_error' or respond == 'request_error':
            print('Retry after 10 seconds.')
            time.sleep(10)
        else:
            time.sleep(10)
            #for i in range(10):
            #    check_manager.edge_clusetr_check(Config_Data,API_Manager_Address,edge_cluster_info, respond)
            return respond
    sys.exit('Edge Node deploying fail.')



import sys, time, logging
from NSX import general_request, vcenter_con
logger = logging.getLogger(__name__)

def object_finder(Config_Data, create_profile, API_Manager_Address, url, create_check = False, **Request_Extension):

    for Try_Count in range(5):
        if 'object_data' in Request_Extension:
            respond = Request_Extension['object_data']
        else:
            respond = general_request.request('GET', Config_Data, API_Manager_Address, url)
        #print(respond)
        if respond == 'respond_error' or respond == 'request_error':
            print('Retry after 10 seconds.')
            time.sleep(10)
        else:
            if 'results' in respond:
                nsx_profiles = respond['results']
            elif 'rules' in respond:
                nsx_profiles = respond['rules']
            else:
                nsx_profiles = [respond]
            #print(respond)
            for nsx_profile in nsx_profiles:
                if 'compute_collection_id' in nsx_profile.keys():
                    for i in range(len(create_profile)):
                        cluster_id = id_finder(Config_Data, API_Manager_Address, 'api/v1/fabric/compute-collections',create_profile[i]['Name'])
                        if nsx_profile['compute_collection_id'] == cluster_id:
                            create_profile[i]['id'] = nsx_profile['id']
                            if create_check == True:
                                print('\033[32m' + create_profile[i]['Name'], ' successfully created.' + '\033[0m')
                                logger.info(f'{create_profile[i]["Name"]} successfully created. ID : {nsx_profile["id"]}')
                            else:
                                print('\033[33m' + f'Required object {create_profile[i]["Name"]} is existed.' + '\033[0m')
                                logger.info(f'Required object {create_profile[i]["Name"]} is existed. ID : {nsx_profile["id"]}')
                        else:
                            pass
                elif 'license_key' in nsx_profile:
                    for i in range(len(create_profile)):
                        if nsx_profile['license_key'] == Config_Data['License'][create_profile[i]['Name']]:
                            if create_check == True:
                                create_profile[i]['license_key'] = nsx_profile['license_key']
                                print('\033[32m' + f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully registered.' + '\033[0m')
                                print(f'Created {create_profile[i]["Object_type"]} : ', create_profile[i]["Name"], '/ License : ', nsx_profile['license_key'])
                                logger.info(f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully registered. ID : {nsx_profile["license_key"]}')
                                create_profile[i]['registered'] = True
                            else:
                                print('\033[33m' + f'Required object {create_profile[i]["Name"]} is existed.'+ '\033[0m')
                                print('Name : ', create_profile[i]["Name"], '/ License : ', nsx_profile['license_key'])
                                logger.info(f'Required object {create_profile[i]["Name"]} is existed. ID : {nsx_profile["license_key"]}')
                                create_profile[i]['registered'] = True
                        else:
                            pass
                        #print(nsx_profile)

                elif 'id' in nsx_profile:
                    for i in range(len(create_profile)):
                        if nsx_profile['display_name'] == create_profile[i]['Name']:
                            create_profile[i]['id'] = nsx_profile['id']
                            if 'path' in nsx_profile:
                                create_profile[i]['path'] = nsx_profile['path']
                            if 'sequence_number' in nsx_profile:
                                create_profile[i]['sequence_number'] = nsx_profile['sequence_number']
                            if create_check == True:
                                #print(nsx_profile)
                                if 'API_MODE' in Request_Extension:
                                    if Request_Extension['API_MODE'] == 'create':
                                        print('\033[32m' + f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully created.' + '\033[0m')
                                        logger.info(f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully create. ID : {nsx_profile["id"]}')
                                    elif Request_Extension['API_MODE'] == 'update':
                                        print('\033[32m' + f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully updated.' + '\033[0m')
                                        logger.info(f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully update. ID : {nsx_profile["id"]}')
                                else:
                                    print('\033[32m' + f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully created.' + '\033[0m')
                                    logger.info(f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully create. ID : {nsx_profile["id"]}')
                                print(f'Created {create_profile[i]["Object_type"]} : ', create_profile[i]["Name"], '/ ID : ', nsx_profile['id'])
                            else:
                                if 'Request_Object_Type' in Request_Extension:
                                    if 'API_MODE' in Request_Extension:
                                        if Request_Extension['API_MODE'] == 'create':
                                            print('\033[33m' + f'object {nsx_profile["display_name"]} {Request_Extension["Request_Object_Type"]} is already created.'+ '\033[0m')
                                            print('Name : ', nsx_profile['display_name'], '/ ID : ', nsx_profile['id'] )
                                            logger.info(f'object {nsx_profile["display_name"]} {Request_Extension["Request_Object_Type"]} is already created. ID : {nsx_profile["id"]}')
                                    else:
                                        print('\033[33m' + f'Required object {nsx_profile["display_name"]} {Request_Extension["Request_Object_Type"]} is existed.'+ '\033[0m')
                                        print('Name : ', nsx_profile['display_name'], '/ ID : ', nsx_profile['id'] )
                                        logger.info(f'Required object {nsx_profile["display_name"]} {Request_Extension["Request_Object_Type"]} is existed. ID : {nsx_profile["id"]}')
                                else:
                                    print('\033[33m' + f'Required object {nsx_profile["display_name"]} is existed.'+ '\033[0m')
                                    print('Name : ', nsx_profile['display_name'], '/ ID : ', nsx_profile['id'] )
                                    logger.info(f'Required object {nsx_profile["display_name"]} is existed. ID : {nsx_profile["id"]}')
                        else:
                            pass
                elif 'external_id' in nsx_profile:
                    for i in range(len(create_profile)):
                        if nsx_profile['display_name'] == create_profile[i]['Name']:
                            create_profile[i]['external_id'] = nsx_profile['external_id']
                            print('\033[33m' + f'Required object {nsx_profile["display_name"]} is existed.'+ '\033[0m')
                            print('Name : ', nsx_profile['display_name'], '/ EXTERNAL ID : ', nsx_profile['external_id'] )
                            logger.info(f'Required object {nsx_profile["display_name"]} is existed. EXTERNAL ID : {nsx_profile["external_id"]}')
                elif 'rule_id' in nsx_profile:
                    for i in range(len(create_profile)):
                        if nsx_profile['display_name'] == create_profile[i]['Name']:
                            create_profile[i]['rule_id'] = nsx_profile['rule_id']
                            if 'sequence_number' in nsx_profile:
                                create_profile[i]['sequence_number'] = nsx_profile['sequence_number']
                            if create_check == True:
                                if Request_Extension['API_MODE'] == 'create':
                                    print('\033[32m' + f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully created.' + '\033[0m')
                                    logger.info(f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully create. ID : {nsx_profile["rule_id"]}')
                                elif Request_Extension['API_MODE'] == 'update':
                                    print('\033[32m' + f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully updated.' + '\033[0m')
                                    logger.info(f'{create_profile[i]["Name"]} {create_profile[i]["Object_type"]} successfully update. ID : {nsx_profile["rule_id"]}')
                            else:
                                if 'Request_Object_Type' in Request_Extension:
                                    if Request_Extension['API_MODE'] == 'create':
                                        print('\033[33m' + f'object {nsx_profile["display_name"]} {Request_Extension["Request_Object_Type"]} is already created.'+ '\033[0m')
                                        print('Name : ', nsx_profile['display_name'], '/ ID : ', nsx_profile['rule_id'] )
                                        logger.info(f'object {nsx_profile["display_name"]} {Request_Extension["Request_Object_Type"]} is already created. ID : {nsx_profile["rule_id"]}')
                                else:
                                    print('\033[33m' + f'Required object {nsx_profile["display_name"]} is existed.'+ '\033[0m')
                                    print('Name : ', nsx_profile['display_name'], '/ ID : ', nsx_profile['id'] )
                                    logger.info(f'Required object {nsx_profile["display_name"]} is existed. ID : {nsx_profile["id"]}')
                        else:
                            pass
            if 'API_MODE' in Request_Extension:
                if Request_Extension['API_MODE'] == 'delete':
                    print('\033[31m' + f'Required object {create_profile[0]["Name"]} successfully deleted.'+ '\033[0m')
                    logger.info(f'Required object {create_profile[0]["Name"]} successfully deleted. NAME : {create_profile[0]["Name"]}')

            return create_profile
    sys.exit('Object Search failed')


def id_finder(Config_Data, API_Manager_Address, Request_URL, Request_Object_Name):
    for Try_Count in range(3):
        respond = general_request.request('GET', Config_Data, API_Manager_Address, Request_URL)

        if respond == 'respond_error' or respond == 'request_error':
            print('Retry after 10 seconds.')
            time.sleep(10)
        else:
            for find_item in respond['results']:
                if find_item['display_name'] == Request_Object_Name:
                    if 'id' in find_item:
                        return find_item['id']
                    elif 'uuid' in find_item:
                        return find_item['uuid']
                    elif 'external_id' in find_item:
                        return find_item['external_id']
                else:pass
    sys.exit('ID Find failed')

def manager_finder(Config_Data, Manager_listup):

    for Config_index in Config_Data:
        if Config_index[:7] == 'vCenter':
            if Config_Data["NSX Manager 01 information"]["vcenter name"] == Config_Data[Config_index]["Name"]:
                vc_session_id = vcenter_con.connect_to_api(Config_Data[Config_index])
                vc_address = Config_Data[Config_index]['ip or FQDN']

    #vm_list = vcenter_search_info(Config_Data, vc_session_id, vc_address, 'vm')
    for Try_Count in range(5):
        vm_list = general_request.request('GET',Config_Data,vc_address,'api/vcenter/vm',Add_header={'vmware-api-session-id':vc_session_id})

        if vm_list == 'respond_error' or vm_list == 'request_error':
            print('Retry after 10 seconds.')
            time.sleep(10)
        else:
            for Manager in Manager_listup:
                #print(Manager)
                result = vcenter_con.find_object(vm_list, Manager["Host Name"],'vm')

                if result == 'object_non':
                    Manager['deployment'] = False
                    #print('\033[32m' + f'{Manager["Host Name"]} Manager is not deployed.' + '\033[0m')
                    logger.info(f'{Manager["Host Name"]} Manager is not deployed.' )
                else:
                    #print(result)
                    for vm in vm_list:
                        if result == vm['vm'] and vm['power_state'] == 'POWERED_OFF':
                            print('\033[31m' + f'{Manager["Host Name"]} is existed. But POWERED OFF.' + '\033[0m')
                            logger.warning((f'{Manager["Host Name"]} is existed. But POWERED OFF.'))
                            sys.exit('script finsh')
                    VM_ip = general_request.request('GET',Config_Data,vc_address,f'api/vcenter/vm/{result}/guest/identity',Add_header={'vmware-api-session-id':vc_session_id})['ip_address']

                    if VM_ip == Manager['Address']:
                        Manager['deployment'] = True
                        #print('\033[31m' + f'{Manager["Host Name"]} Manager already existed.' + '\033[0m')
                        logger.warning(f'{Manager["Host Name"]} Manager already existed.')
                    else:
                        print('\033[33m' + f'{Manager["Host Name"]} Manager already existed. But ip address is different.' + '\033[0m')
                        logger.warning(f'{Manager["Host Name"]} Manager already existed. But ip address is different.')
                        sys.exit('CHECK YOUR vCenter.')
    return Manager_listup

def lb_object_search(Config_Data,search_object,API_Manager_Address, object_type):
    print(f'{search_object["Name"]} {object_type} Checking...')
    logger.info(f'{search_object["Name"]} {object_type} Checking...')
    if object_type == 'Load Balancer':
        url = 'lb-services'
    elif object_type == 'Application Profile':
        url = 'lb-app-profiles'
    elif object_type == 'Persistence Profile':
        url = 'lb-persistence-profiles'
    elif object_type == 'Server Pool':
        url = 'lb-pools'
    elif object_type == 'SSL Client Certificate Profile':
        url = 'lb-client-ssl-profiles'
    elif object_type == 'SSL Certificate':
        url = 'certificates'
    elif object_type == 'SSL Server Certificate Profile':
        url = 'lb-server-ssl-profiles'
    elif object_type == 'Active Monitor':
        url = 'lb-monitor-profiles'
    #print(url)

    object_id = object_finder(Config_Data,[{'Name':search_object["Name"]}],API_Manager_Address,f'policy/api/v1/infra/{url}')[0]
    if 'id' in object_id:
        return object_id['id']
    else:
        logger.warning('\033[32m' + f'{search_object["Name"]} {object_type} not found.' + '\033[0m')
        sys.exit(f'{search_object["Name"]} {object_type} not found.')
